#!/usr/bin/env python3

import enum


class error(enum.IntEnum):
    """Error codes exported from Zephyr `hci_types.h`"""

    SUCCESS = (0x00, "Success")
    UNKNOWN_CMD = (0x01, "UNKNOWN HCI COMMAND")
    UNKNOWN_CONN_ID = (0x02, "UNKNOWN CONNECTION IDENTIFIER")
    HW_FAILURE = (0x03, "HARDWARE FAILURE")
    PAGE_TIMEOUT = (0x04, "PAGE TIMEOUT")
    AUTH_FAIL = (0x05, "AUTHENTICATION FAILURE")
    PIN_OR_KEY_MISSING = (0x06, "PIN OR KEY MISSING")
    MEM_CAPACITY_EXCEEDED = (0x07, "MEMORY CAPACITY EXCEEDED")
    CONN_TIMEOUT = (0x08, "CONNECTION TIMEOUT")
    CONN_LIMIT_EXCEEDED = (0x09, "CONNECTION LIMIT EXCEEDED")
    SYNC_CONN_LIMIT_EXCEEDED = (0x0A, "SYNCHRONOUS CONNECTION LIMIT TO A DEVICE EXCEEDED")
    CONN_ALREADY_EXISTS = (0x0B, "CONNECTION ALREADY EXISTS")
    CMD_DISALLOWED = (0x0C, "COMMAND DISALLOWED")
    INSUFFICIENT_RESOURCES = (0x0D, "CONNECTION REJECTED DUE TO LIMITED RESOURCES")
    INSUFFICIENT_SECURITY = (0x0E, "CONNECTION REJECTED DUE TO SECURITY REASONS")
    BD_ADDR_UNACCEPTABLE = (0x0F, "CONNECTION REJECTED DUE TO UNACCEPTABLE BD_ADDR")
    CONN_ACCEPT_TIMEOUT = (0x10, "CONNECTION ACCEPT TIMEOUT EXCEEDED")
    UNSUPP_FEATURE_PARAM_VAL = (0x11, "UNSUPPORTED FEATURE OR PARAMETER VALUE")
    INVALID_PARAM = (0x12, "INVALID HCI COMMAND PARAMETERS")
    REMOTE_USER_TERM_CONN = (0x13, "REMOTE USER TERMINATED CONNECTION")
    REMOTE_LOW_RESOURCES = (0x14, "REMOTE DEVICE TERMINATED CONNECTION DUE TO LOW RESOURCES")
    REMOTE_POWER_OFF = (0x15, "REMOTE DEVICE TERMINATED CONNECTION DUE TO POWER OFF")
    LOCALHOST_TERM_CONN = (0x16, "CONNECTION TERMINATED BY LOCAL HOST")
    REPEATED_ATTEMPTS = (0x17, "REPEATED ATTEMPTS")
    PAIRING_NOT_ALLOWED = (0x18, "PAIRING NOT ALLOWED")
    UNKNOWN_LMP_PDU = (0x19, "UNKNOWN LMP PDU")
    UNSUPP_REMOTE_FEATURE = (0x1A, "UNSUPPORTED REMOTE FEATURE")
    SCO_OFFSET_REJECTED = (0x1B, "SCO OFFSET REJECTED")
    SCO_INTERVAL_REJECTED = (0x1C, "SCO INTERVAL REJECTED")
    SCO_AIR_MODE_REJECTED = (0x1D, "SCO AIR MODE REJECTED")
    INVALID_LL_PARAM = (0x1E, "INVALID LL PARAMETERS")
    UNSPECIFIED = (0x1F, "UNSPECIFIED ERROR")
    UNSUPP_LL_PARAM_VAL = (0x20, "UNSUPPORTED LL PARAMETER VALUE")
    ROLE_CHANGE_NOT_ALLOWED = (0x21, "ROLE CHANGE NOT ALLOWED")
    LL_RESP_TIMEOUT = (0x22, "LL RESPONSE TIMEOUT")
    LL_PROC_COLLISION = (0x23, "LL PROCEDURE COLLISION")
    LMP_PDU_NOT_ALLOWED = (0x24, "LMP PDU NOT ALLOWED")
    ENC_MODE_NOT_ACCEPTABLE = (0x25, "ENCRYPTION MODE NOT ACCEPTABLE")
    LINK_KEY_CANNOT_BE_CHANGED = (0x26, "LINK KEY CANNOT BE CHANGED")
    REQUESTED_QOS_NOT_SUPPORTED = (0x27, "REQUESTED QoS NOT SUPPORTED")
    INSTANT_PASSED = (0x28, "INSTANT PASSED")
    PAIRING_NOT_SUPPORTED = (0x29, "PAIRING WITH UNIT KEY NOT SUPPORTED")
    DIFF_TRANS_COLLISION = (0x2A, "DIFFERENT TRANSACTION COLLISION")
    QOS_UNACCEPTABLE_PARAM = (0x2C, "QoS UNACCEPTABLE PARAMETER")
    QOS_REJECTED = (0x2D, "QoS REJECTED")
    CHAN_ASSESS_NOT_SUPPORTED = (0x2E, "CHANNEL ASSESSMENT NOT SUPPORTED")
    INSUFF_SECURITY = (0x2F, "INSUFFICIENT SECURITY")
    PARAM_OUT_OF_MANDATORY_RANGE = (0x30, "PARAMETER OUT OF MANDATORY RANGE")
    ROLE_SWITCH_PENDING = (0x32, "ROLE SWITCH PENDING")
    RESERVED_SLOT_VIOLATION = (0x34, "RESERVED SLOT VIOLATION")
    ROLE_SWITCH_FAILED = (0x35, "ROLE SWITCH FAILED")
    EXT_INQ_RESP_TOO_LARGE = (0x36, "EXTENDED INQUIRY RESPONSE TOO LARGE")
    SIMPLE_PAIR_NOT_SUPP_BY_HOST = (0x37, "SECURE SIMPLE PAIRING NOT SUPPORTED BY HOST")
    HOST_BUSY_PAIRING = (0x38, "HOST BUSY–PAIRING")
    CONN_REJECTED_DUE_TO_NO_CHAN = (0x39, "CONNECTION REJECTED DUE TO NO SUITABLE CHANNEL FOUND")
    CONTROLLER_BUSY = (0x3A, "CONTROLLER BUSY")
    UNACCEPT_CONN_PARAM = (0x3B, "UNACCEPTABLE CONNECTION PARAMETERS")
    ADV_TIMEOUT = (0x3C, "ADVERTISING TIMEOUT")
    TERM_DUE_TO_MIC_FAIL = (0x3D, "CONNECTION TERMINATED DUE TO MIC FAILURE")
    CONN_FAIL_TO_ESTAB = (0x3E, "CONNECTION FAILED TO BE ESTABLISHED")
    MAC_CONN_FAILED = (0x3F, "LEGACY ERROR")
    CLOCK_ADJUST_REJECTED = (0x40, "COARSE CLOCK ADJUSTMENT REJECTED")
    SUBMAP_NOT_DEFINED = (0x41, "TYPE0 SUBMAP NOT DEFINED")
    UNKNOWN_ADV_IDENTIFIER = (0x42, "UNKNOWN ADVERTISING IDENTIFIER")
    LIMIT_REACHED = (0x43, "LIMIT REACHED")
    OP_CANCELLED_BY_HOST = (0x44, "OPERATION CANCELLED BY HOST")
    PACKET_TOO_LONG = (0x45, "PACKET TOO LONG")
    TOO_LATE = (0x46, "TOO LATE")
    TOO_EARLY = (0x47, "TOO EARLY")

    description: str

    def __new__(cls, value: int, description: str = ""):
        obj = int.__new__(cls, value)
        obj._value_ = value
        obj.description = description
        return obj

    @classmethod
    def strerror(cls, int) -> str:
        return cls(int).description



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.route_type import RouteType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.algorithm import Algorithm
    from ..models.application_version import ApplicationVersion


T = TypeVar("T", bound="DeviceState")


@_attrs_define
class DeviceState:
    """
    Attributes:
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        application_id (Union[Unset, int]): Last announced application ID
        application_version (Union[Unset, ApplicationVersion]): Application version
        algorithms (Union[Unset, list['Algorithm']]): Last announced algorithms
        last_route_interface (Union[Unset, RouteType]): Interface of route
        last_route_udp_address (Union[Unset, str]): UDP address of last packet sent by device
    """

    created_at: datetime.datetime
    updated_at: datetime.datetime
    application_id: Unset | int = UNSET
    application_version: Union[Unset, "ApplicationVersion"] = UNSET
    algorithms: Unset | list["Algorithm"] = UNSET
    last_route_interface: Unset | RouteType = UNSET
    last_route_udp_address: Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        application_id = self.application_id

        application_version: Unset | dict[str, Any] = UNSET
        if not isinstance(self.application_version, Unset):
            application_version = self.application_version.to_dict()

        algorithms: Unset | list[dict[str, Any]] = UNSET
        if not isinstance(self.algorithms, Unset):
            algorithms = []
            for algorithms_item_data in self.algorithms:
                algorithms_item = algorithms_item_data.to_dict()
                algorithms.append(algorithms_item)

        last_route_interface: Unset | str = UNSET
        if not isinstance(self.last_route_interface, Unset):
            last_route_interface = self.last_route_interface.value

        last_route_udp_address = self.last_route_udp_address

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )
        if application_id is not UNSET:
            field_dict["applicationId"] = application_id
        if application_version is not UNSET:
            field_dict["applicationVersion"] = application_version
        if algorithms is not UNSET:
            field_dict["algorithms"] = algorithms
        if last_route_interface is not UNSET:
            field_dict["lastRouteInterface"] = last_route_interface
        if last_route_udp_address is not UNSET:
            field_dict["lastRouteUdpAddress"] = last_route_udp_address

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.algorithm import Algorithm
        from ..models.application_version import ApplicationVersion

        d = dict(src_dict)
        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        application_id = d.pop("applicationId", UNSET)

        _application_version = d.pop("applicationVersion", UNSET)
        application_version: Unset | ApplicationVersion
        if isinstance(_application_version, Unset):
            application_version = UNSET
        else:
            application_version = ApplicationVersion.from_dict(_application_version)

        algorithms = []
        _algorithms = d.pop("algorithms", UNSET)
        for algorithms_item_data in _algorithms or []:
            algorithms_item = Algorithm.from_dict(algorithms_item_data)

            algorithms.append(algorithms_item)

        _last_route_interface = d.pop("lastRouteInterface", UNSET)
        last_route_interface: Unset | RouteType
        if isinstance(_last_route_interface, Unset):
            last_route_interface = UNSET
        else:
            last_route_interface = RouteType(_last_route_interface)

        last_route_udp_address = d.pop("lastRouteUdpAddress", UNSET)

        device_state = cls(
            created_at=created_at,
            updated_at=updated_at,
            application_id=application_id,
            application_version=application_version,
            algorithms=algorithms,
            last_route_interface=last_route_interface,
            last_route_udp_address=last_route_udp_address,
        )

        device_state.additional_properties = d
        return device_state

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.device_entry_update_status import DeviceEntryUpdateStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.new_device_kv_entry_update_decoded import NewDeviceKVEntryUpdateDecoded


T = TypeVar("T", bound="DeviceKVEntryUpdate")


@_attrs_define
class DeviceKVEntryUpdate:
    """
    Attributes:
        id (UUID): ID of update
        key_id (int): Key id
        crc (int): CRC32 of entry update value
        status (DeviceEntryUpdateStatus): Status of device KV entry update
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        data (Union[Unset, str]): Raw entry data as a base64 encoded string (must provide either data or decoded)
        decoded (Union[Unset, NewDeviceKVEntryUpdateDecoded]): Decoded entry value (must provide either data or decoded)
        last_error (Union[Unset, str]): Last error message if update failed
        last_attempt_at (Union[Unset, datetime.datetime]): Time of last attempt
    """

    id: UUID
    key_id: int
    crc: int
    status: DeviceEntryUpdateStatus
    created_at: datetime.datetime
    updated_at: datetime.datetime
    data: Unset | str = UNSET
    decoded: Union[Unset, "NewDeviceKVEntryUpdateDecoded"] = UNSET
    last_error: Unset | str = UNSET
    last_attempt_at: Unset | datetime.datetime = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = str(self.id)

        key_id = self.key_id

        crc = self.crc

        status = self.status.value

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        data = self.data

        decoded: Unset | dict[str, Any] = UNSET
        if not isinstance(self.decoded, Unset):
            decoded = self.decoded.to_dict()

        last_error = self.last_error

        last_attempt_at: Unset | str = UNSET
        if not isinstance(self.last_attempt_at, Unset):
            last_attempt_at = self.last_attempt_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "keyId": key_id,
                "crc": crc,
                "status": status,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data
        if decoded is not UNSET:
            field_dict["decoded"] = decoded
        if last_error is not UNSET:
            field_dict["lastError"] = last_error
        if last_attempt_at is not UNSET:
            field_dict["lastAttemptAt"] = last_attempt_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.new_device_kv_entry_update_decoded import NewDeviceKVEntryUpdateDecoded

        d = dict(src_dict)
        id = UUID(d.pop("id"))

        key_id = d.pop("keyId")

        crc = d.pop("crc")

        status = DeviceEntryUpdateStatus(d.pop("status"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        data = d.pop("data", UNSET)

        _decoded = d.pop("decoded", UNSET)
        decoded: Unset | NewDeviceKVEntryUpdateDecoded
        if isinstance(_decoded, Unset):
            decoded = UNSET
        else:
            decoded = NewDeviceKVEntryUpdateDecoded.from_dict(_decoded)

        last_error = d.pop("lastError", UNSET)

        _last_attempt_at = d.pop("lastAttemptAt", UNSET)
        last_attempt_at: Unset | datetime.datetime
        if isinstance(_last_attempt_at, Unset):
            last_attempt_at = UNSET
        else:
            last_attempt_at = isoparse(_last_attempt_at)

        device_kv_entry_update = cls(
            id=id,
            key_id=key_id,
            crc=crc,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            data=data,
            decoded=decoded,
            last_error=last_error,
            last_attempt_at=last_attempt_at,
        )

        device_kv_entry_update.additional_properties = d
        return device_kv_entry_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.definitions_enum_value import DefinitionsEnumValue


T = TypeVar("T", bound="DefinitionsEnumDefinition")


@_attrs_define
class DefinitionsEnumDefinition:
    """
    Attributes:
        description (str):
        type_ (str):
        values (list['DefinitionsEnumValue']):
    """

    description: str
    type_: str
    values: list["DefinitionsEnumValue"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        description = self.description

        type_ = self.type_

        values = []
        for values_item_data in self.values:
            values_item = values_item_data.to_dict()
            values.append(values_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "description": description,
                "type": type_,
                "values": values,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.definitions_enum_value import DefinitionsEnumValue

        d = dict(src_dict)
        description = d.pop("description")

        type_ = d.pop("type")

        values = []
        _values = d.pop("values")
        for values_item_data in _values:
            values_item = DefinitionsEnumValue.from_dict(values_item_data)

            values.append(values_item)

        definitions_enum_definition = cls(
            description=description,
            type_=type_,
            values=values,
        )

        definitions_enum_definition.additional_properties = d
        return definitions_enum_definition

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="GeneratedMQTTToken")


@_attrs_define
class GeneratedMQTTToken:
    """
    Attributes:
        token (str): Generated MQTT token
        issued_at (datetime.datetime): Issue time of token
        expires_at (datetime.datetime): Expiry time of token
    """

    token: str
    issued_at: datetime.datetime
    expires_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        token = self.token

        issued_at = self.issued_at.isoformat()

        expires_at = self.expires_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "token": token,
                "issuedAt": issued_at,
                "expiresAt": expires_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        token = d.pop("token")

        issued_at = isoparse(d.pop("issuedAt"))

        expires_at = isoparse(d.pop("expiresAt"))

        generated_mqtt_token = cls(
            token=token,
            issued_at=issued_at,
            expires_at=expires_at,
        )

        generated_mqtt_token.additional_properties = d
        return generated_mqtt_token

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3

import os
import pathlib
import subprocess

import pytest

import infuse_iot.credentials as cred

assert "TOXTEMPDIR" in os.environ, "you must run these tests using tox"


def test_custom_tool_integration():
    # Validate custom tool integration
    echo_string = "test_string"

    try:
        cred.delete_custom_tool_path()
    except Exception as _:
        pass

    with pytest.raises(subprocess.CalledProcessError):
        subprocess.check_output(["infuse", "custom_tool", "--echo", echo_string])

    custom_tools_path = pathlib.Path(__file__).parent.parent / 'scripts' / 'custom_tools'

    subprocess.check_output(["infuse", "credentials", "--custom-tools", str(custom_tools_path)])

    output = subprocess.check_output(["infuse", "custom_tool", "--echo", echo_string]).decode()
    assert echo_string in output

    cred.delete_custom_tool_path()

    with pytest.raises(subprocess.CalledProcessError):
        subprocess.check_output(["infuse", "custom_tool", "--echo", echo_string])



#!/usr/bin/env python3

import ctypes
from collections.abc import Generator
from typing import Any, cast

from typing_extensions import Self


def _public_name(internal_field):
    if internal_field[0][0] == "_":
        return internal_field[0][1:]
    return internal_field[0]


class TdfField:
    def __init__(self, field: str, subfield: str | None, postfix: str, display_fmt: str, val: Any):
        self.field = field
        self.subfield = subfield
        self.postfix = postfix
        self._display_fmt = display_fmt
        self.val = val

    @property
    def name(self) -> str:
        if self.subfield:
            return f"{self.field}.{self.subfield}"
        return self.field

    def val_fmt(self) -> str:
        if isinstance(self.val, list) and self._display_fmt != "{}":
            return ",".join([self._display_fmt.format(v) for v in self.val])
        else:
            return self._display_fmt.format(self.val)


class TdfStructBase(ctypes.LittleEndianStructure):
    def iter_fields(
        self,
        field: str,
    ) -> Generator[TdfField, None, None]:
        for subfield in self._fields_:
            sf_name = _public_name(subfield)
            val = getattr(self, sf_name)
            yield TdfField(
                field,
                sf_name,
                self._postfix_[sf_name],
                self._display_fmt_[sf_name],
                val,
            )


class TdfReadingBase(ctypes.LittleEndianStructure):
    NAME: str
    ID: int

    def iter_fields(self, nested_iter: bool = True) -> Generator[TdfField, None, None]:
        for field in self._fields_:
            f_name = _public_name(field)
            val = getattr(self, f_name)
            if nested_iter and isinstance(val, TdfStructBase):
                yield from val.iter_fields(f_name)
            else:
                if isinstance(val, ctypes.Array):
                    val = list(val)
                yield TdfField(
                    f_name,
                    None,
                    self._postfix_[f_name],
                    self._display_fmt_[f_name],
                    val,
                )

    @classmethod
    def field_information(cls):
        info = []
        for field in cls._fields_:
            f_name = _public_name(field)

            if issubclass(field[1], ctypes.LittleEndianStructure):
                subfields = []
                for subfield in field[1]._fields_:
                    sf_name = _public_name(subfield)
                    subfields.append({"name": sf_name, "type": subfield[1]})
                info.append({"name": f_name, "type": field[1], "subfields": subfields})
            elif isinstance(field[1], ctypes.Array):
                info.append({"name": f_name, "type": list})
            else:
                info.append({"name": f_name, "type": field[1]})
        return info

    @classmethod
    def from_buffer_consume(cls, source: bytes, offset: int = 0) -> Self:
        last_field = cls._fields_[-1]

        # Last value not a VLA
        if not issubclass(last_field[1], ctypes.Array):
            return cls.from_buffer_copy(source, offset)
        last_field_type: ctypes.Array = last_field[1]  # type: ignore
        if last_field_type._length_ != 0:
            return cls.from_buffer_copy(source, offset)

        base_size = ctypes.sizeof(cls)
        var_name = last_field[0]
        var_type = last_field_type._type_
        var_type_size = ctypes.sizeof(var_type)

        source_var_len = len(source) - base_size
        if source_var_len % var_type_size != 0:
            raise RuntimeError
        source_var_num = source_var_len // var_type_size

        # No trailing VLA
        if source_var_num == 0:
            return cls.from_buffer_copy(source, offset)

        # Dynamically create subclass with correct length
        class TdfVLA(ctypes.LittleEndianStructure):
            NAME = cls.NAME
            ID = cls.ID
            _fields_ = cls._fields_[:-1] + [(var_name, source_var_num * var_type)]  # type: ignore
            _pack_ = 1
            _postfix_ = cls._postfix_
            _display_fmt_ = cls._display_fmt_
            _vla_field_ = var_name
            iter_fields = cls.iter_fields
            field_information = cls.field_information

        # Copy convertor functions for fields
        for f in cls._fields_:
            if f[0][0] == "_":
                f_name = f[0][1:]
                setattr(TdfVLA, f_name, getattr(cls, f_name))

        # Create the object instance
        return cast(Self, TdfVLA.from_buffer_copy(source, offset))



#!/usr/bin/env python3

"""Annotate CSV data"""

__author__ = "Jordan Yates"
__copyright__ = "Copyright 2024, Embeint Holdings Pty Ltd"

from infuse_iot.commands import InfuseCommand
from infuse_iot.util.argparse import ValidFile


class SubCommand(InfuseCommand):
    NAME = "csv_annotate"
    HELP = "Annotate CSV data"
    DESCRIPTION = "Annotate CSV data"

    @classmethod
    def add_parser(cls, parser):
        parser.add_argument("--file", "-f", required=True, type=ValidFile)
        parser.add_argument("--default", "-d", type=str, default="N/A", help="Default label")

    def __init__(self, args):
        self.file = args.file
        self.labels = [args.default]
        self.selection = []

    def make_plots(self):
        import plotly.graph_objects as go
        from plotly.subplots import make_subplots

        fig = make_subplots(rows=2, cols=1, shared_xaxes=True, vertical_spacing=0.2)
        for col in self.df.columns.values[1:]:
            fig.add_trace(
                go.Scatter(x=self.df["time"], y=self.df[col], name=col),
                row=2 if col == "labels" else 1,
                col=1,
            )

        return fig

    def run(self):
        import pandas as pd
        from dash import Dash, Input, Output, State, callback, dcc, html

        # Read data, add label column
        self.df = pd.read_csv(self.file, parse_dates=["time"])
        self.df.insert(
            len(self.df.columns),
            "labels",
            [self.labels[0] for _ in range(self.df.shape[0])],
        )

        app = Dash()
        app.layout = html.Div(
            [
                dcc.Graph(id="graph", figure=self.make_plots()),
                html.Button("Label selection", id="button-label-selection", n_clicks=0),
                html.Div(
                    [
                        dcc.Input(id="input-on-submit", type="text"),
                        html.Button("Add new label", id="button-label-add"),
                    ]
                ),
                dcc.RadioItems(id="label-current", options=self.labels),
                html.Button("Remove label", id="button-label-remove"),
            ]
        )

        @callback(
            Output("label-current", "options", True),
            Input("button-label-add", "n_clicks"),
            State("input-on-submit", "value"),
            prevent_initial_call=True,
        )
        def label_add(n_clicks, value):
            self.labels.append(value)
            return self.labels

        @callback(
            Output("label-current", "options", True),
            Input("button-label-remove", "n_clicks"),
            State("label-current", "value"),
            prevent_initial_call=True,
        )
        def label_remove(n_clicks, value):
            try:
                self.labels.remove(value)
            except ValueError:
                pass
            return self.labels

        @callback(Input("graph", "relayoutData"))
        def store_relayout_data(relayoutData):
            if relayoutData.get("autosize", False) or relayoutData.get("xaxis.autorange", False):
                self.selection = [
                    self.df["time"][0],
                    self.df["time"][self.df.shape[0] - 1],
                ]
            else:
                self.selection = [
                    pd.Timestamp(relayoutData["xaxis.range[0]"]),
                    pd.Timestamp(relayoutData["xaxis.range[1]"]),
                ]

        @callback(
            Output("graph", "figure"),
            Input("button-label-selection", "n_clicks"),
            State("label-current", "value"),
            prevent_initial_call=True,
        )
        def label_current_selection(n_clicks, value):
            self.df.loc[
                (self.df["time"] >= self.selection[0]) & (self.df["time"] <= self.selection[1]),
                "labels",
            ] = value
            return self.make_plots()

        app.run_server(debug=True)



#!/usr/bin/env python3

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.ciphers.aead import ChaCha20Poly1305
from cryptography.hazmat.primitives.kdf.hkdf import HKDF


def hkdf_derive(input_key: bytes, salt: bytes, info: bytes) -> bytes:
    """Derive a cryptographic key using HKDF-SHA256"""
    hkdf = HKDF(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        info=info,
    )
    return hkdf.derive(input_key)


def chachapoly_encrypt(key: bytes, associated_data: bytes | None, nonce: bytes, payload: bytes) -> bytes:
    """Encrypt a payload using ChaCha20-Poly1305"""
    cipher = ChaCha20Poly1305(key)
    return cipher.encrypt(nonce, payload, associated_data)


def chachapoly_decrypt(key: bytes, associated_data: bytes | None, nonce: bytes, payload: bytes) -> bytes:
    """Decrypt a payload using ChaCha20-Poly1305"""
    cipher = ChaCha20Poly1305(key)
    return cipher.decrypt(nonce, payload, associated_data)



#!/usr/bin/env python3
# mypy: ignore-errors

"""Autogenerated RPC definitions"""

import ctypes
import enum

from infuse_iot.util.ctypes import VLACompatLittleEndianStruct


class rpc_struct_mcuboot_img_sem_ver(VLACompatLittleEndianStruct):
    """MCUboot semantic versioning struct"""

    _fields_ = [
        ("major", ctypes.c_uint8),
        ("minor", ctypes.c_uint8),
        ("revision", ctypes.c_uint16),
        ("build_num", ctypes.c_uint32),
    ]
    _pack_ = 1


class rpc_struct_kv_store_value(VLACompatLittleEndianStruct):
    """KV store data value"""

    _fields_ = [
        ("id", ctypes.c_uint16),
        ("len", ctypes.c_int16),
    ]
    vla_field = ("data", 0 * ctypes.c_uint8)
    vla_counted_by = "len"
    _pack_ = 1


class rpc_struct_kv_store_crc(VLACompatLittleEndianStruct):
    """KV store data CRC"""

    _fields_ = [
        ("id", ctypes.c_uint16),
        ("crc", ctypes.c_uint32),
    ]
    _pack_ = 1


class rpc_struct_bt_addr_le(VLACompatLittleEndianStruct):
    """Bluetooth LE address"""

    _fields_ = [
        ("type", ctypes.c_uint8),
        ("val", 6 * ctypes.c_uint8),
    ]
    _pack_ = 1


class rpc_struct_ipv4_address(VLACompatLittleEndianStruct):
    """IPv4 address"""

    _fields_ = [
        ("addr", 4 * ctypes.c_uint8),
    ]
    _pack_ = 1


class rpc_struct_ipv6_address(VLACompatLittleEndianStruct):
    """IPv6 address"""

    _fields_ = [
        ("addr", 16 * ctypes.c_uint8),
    ]
    _pack_ = 1


class rpc_struct_network_state(VLACompatLittleEndianStruct):
    """Common network state"""

    _fields_ = [
        ("state", ctypes.c_uint8),
        ("if_flags", ctypes.c_uint32),
        ("l2_flags", ctypes.c_uint16),
        ("mtu", ctypes.c_uint16),
        ("ipv4", rpc_struct_ipv4_address),
        ("ipv6", rpc_struct_ipv6_address),
    ]
    _pack_ = 1


class rpc_struct_wifi_state(VLACompatLittleEndianStruct):
    """WiFi interface status"""

    _fields_ = [
        ("state", ctypes.c_uint8),
        ("ssid", 32 * ctypes.c_char),
        ("bssid", 6 * ctypes.c_uint8),
        ("band", ctypes.c_uint8),
        ("channel", ctypes.c_uint8),
        ("iface_mode", ctypes.c_uint8),
        ("link_mode", ctypes.c_uint8),
        ("security", ctypes.c_uint8),
        ("rssi", ctypes.c_int8),
        ("beacon_interval", ctypes.c_uint16),
        ("twt_capable", ctypes.c_uint8),
    ]
    _pack_ = 1


class rpc_struct_lte_state(VLACompatLittleEndianStruct):
    """LTE interface status"""

    _fields_ = [
        ("registration_state", ctypes.c_uint8),
        ("access_technology", ctypes.c_uint8),
        ("mcc", ctypes.c_uint16),
        ("mnc", ctypes.c_uint16),
        ("cell_id", ctypes.c_uint32),
        ("tac", ctypes.c_uint32),
        ("tau", ctypes.c_int32),
        ("earfcn", ctypes.c_uint16),
        ("band", ctypes.c_uint8),
        ("psm_active_time", ctypes.c_int16),
        ("edrx_interval", ctypes.c_float),
        ("edrx_paging_window", ctypes.c_float),
        ("rsrp", ctypes.c_int16),
        ("rsrq", ctypes.c_int8),
    ]
    _pack_ = 1


class rpc_struct_lte_state_v2(VLACompatLittleEndianStruct):
    """LTE interface status"""

    _fields_ = [
        ("registration_state", ctypes.c_uint8),
        ("access_technology", ctypes.c_uint8),
        ("mcc", ctypes.c_uint16),
        ("mnc", ctypes.c_uint16),
        ("cell_id", ctypes.c_uint32),
        ("tac", ctypes.c_uint32),
        ("tau", ctypes.c_int32),
        ("earfcn", ctypes.c_uint16),
        ("band", ctypes.c_uint8),
        ("psm_active_time", ctypes.c_int16),
        ("edrx_interval", ctypes.c_float),
        ("edrx_paging_window", ctypes.c_float),
        ("rsrp", ctypes.c_int16),
        ("rsrq", ctypes.c_int8),
        ("as_rai", ctypes.c_uint8),
        ("cp_rai", ctypes.c_uint8),
    ]
    _pack_ = 1


class rpc_struct_wifi_scan_result(VLACompatLittleEndianStruct):
    """WiFi interface status"""

    _fields_ = [
        ("band", ctypes.c_uint8),
        ("channel", ctypes.c_uint8),
        ("security", ctypes.c_uint8),
        ("rssi", ctypes.c_int8),
        ("bssid", 6 * ctypes.c_uint8),
        ("ssid_len", ctypes.c_uint8),
    ]
    vla_field = ("ssid", 0 * ctypes.c_char)
    vla_counted_by = "ssid_len"
    _pack_ = 1


class rpc_struct_xyz_s16(VLACompatLittleEndianStruct):
    """Signed 16 bit XYZ vector"""

    _fields_ = [
        ("x", ctypes.c_int16),
        ("y", ctypes.c_int16),
        ("z", ctypes.c_int16),
    ]
    _pack_ = 1


class rpc_struct_infuse_state(VLACompatLittleEndianStruct):
    """Single Infuse-IoT application state"""

    _fields_ = [
        ("state", ctypes.c_uint8),
        ("timeout", ctypes.c_uint16),
    ]
    _pack_ = 1


class rpc_struct_sockaddr(VLACompatLittleEndianStruct):
    """`struct sockaddr_in` or `struct sockaddr_in6` compatible address"""

    _fields_ = [
        ("sin_family", ctypes.c_uint8),
        ("sin_port", ctypes.c_uint16),
        ("sin_addr", 16 * ctypes.c_uint8),
        ("scope_id", ctypes.c_uint8),
    ]
    _pack_ = 1


class rpc_struct_heap_info(VLACompatLittleEndianStruct):
    """struct k_heap information"""

    _fields_ = [
        ("addr", ctypes.c_uint32),
        ("free_bytes", ctypes.c_uint32),
        ("allocated_bytes", ctypes.c_uint32),
        ("max_allocated_bytes", ctypes.c_uint32),
    ]
    _pack_ = 1


class rpc_enum_bt_le_addr_type(enum.IntEnum):
    """Bluetooth LE address type"""

    PUBLIC = 0
    RANDOM = 1


class rpc_enum_file_action(enum.IntEnum):
    """Actions to take upon receiving a file"""

    DISCARD = 0
    APP_IMG = 1
    BT_CTLR_IMG = 2
    APP_CPATCH = 11
    BT_CTLR_CPATCH = 12
    NRF91_MODEM_DIFF = 20
    FILE_FOR_COPY = 30


class rpc_enum_infuse_bt_characteristic(enum.IntEnum):
    """Infuse-IoT Bluetooth characteristics (Bitmask)"""

    COMMAND = 1
    DATA = 2
    LOGGING = 4


class rpc_enum_data_logger(enum.IntEnum):
    """Data Logger identifier"""

    FLASH_ONBOARD = 1
    FLASH_REMOVABLE = 2
    UDP = 3


class rpc_enum_zperf_data_source(enum.IntEnum):
    """Source for zperf data upload"""

    CONSTANT = 0
    RANDOM = 1
    FLASH_ONBOARD = 2
    FLASH_REMOVABLE = 3
    ENCRYPT = 128


class rpc_enum_key_id(enum.IntEnum):
    """Infuse security key identifier"""

    NETWORK_KEY = 0
    SECONDARY_NETWORK_KEY = 1


class rpc_enum_key_action(enum.IntEnum):
    """Infuse security key action"""

    KEY_WRITE = 0
    KEY_DELETE = 1


class rpc_enum_support_status(enum.IntEnum):
    """Field support status"""

    UNSUPPORTED = 0
    SUPPORTED = 1
    UNKNOWN = 255


class RPCDefinitionBase:
    NAME: str
    HELP: str
    DESCRIPTION: str
    COMMAND_ID: int


class reboot(RPCDefinitionBase):
    """Reboot the device after a delay"""

    NAME = "reboot"
    HELP = "Reboot the device after a delay"
    DESCRIPTION = "Reboot the device after a delay"
    COMMAND_ID = 1

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("delay_ms", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("delay_ms", ctypes.c_uint32),
        ]
        _pack_ = 1


class fault(RPCDefinitionBase):
    """Immediately trigger an exception on the device"""

    NAME = "fault"
    HELP = "Immediately trigger an exception on the device"
    DESCRIPTION = "Immediately trigger an exception on the device"
    COMMAND_ID = 2

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("fault", ctypes.c_uint8),
            ("zero", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class time_get(RPCDefinitionBase):
    """Get the current time knowledge of the device"""

    NAME = "time_get"
    HELP = "Get the current time knowledge of the device"
    DESCRIPTION = "Get the current time knowledge of the device"
    COMMAND_ID = 3

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("time_source", ctypes.c_uint8),
            ("epoch_time", ctypes.c_uint64),
            ("sync_age", ctypes.c_uint32),
        ]
        _pack_ = 1


class time_set(RPCDefinitionBase):
    """Set the current time of the device"""

    NAME = "time_set"
    HELP = "Set the current time of the device"
    DESCRIPTION = "Set the current time of the device"
    COMMAND_ID = 4

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("epoch_time", ctypes.c_uint64),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class kv_write(RPCDefinitionBase):
    """Write values to the KV store"""

    NAME = "kv_write"
    HELP = "Write values to the KV store"
    DESCRIPTION = "Write values to the KV store"
    COMMAND_ID = 5

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("num", ctypes.c_uint8),
        ]
        vla_field = ("values", 0 * rpc_struct_kv_store_value)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("rc", 0 * ctypes.c_int16)
        _pack_ = 1


class kv_read(RPCDefinitionBase):
    """Read values from the KV store"""

    NAME = "kv_read"
    HELP = "Read values from the KV store"
    DESCRIPTION = "Read values from the KV store"
    COMMAND_ID = 6

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("num", ctypes.c_uint8),
        ]
        vla_field = ("keys", 0 * ctypes.c_uint16)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("values", 0 * rpc_struct_kv_store_value)
        _pack_ = 1


class kv_reflect_crcs(RPCDefinitionBase):
    """Read KV store CRCs"""

    NAME = "kv_reflect_crcs"
    HELP = "Read KV store CRCs"
    DESCRIPTION = "Read KV store CRCs"
    COMMAND_ID = 7

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("offset", ctypes.c_uint16),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("num", ctypes.c_uint16),
            ("remaining", ctypes.c_uint16),
        ]
        vla_field = ("crcs", 0 * rpc_struct_kv_store_crc)
        _pack_ = 1


class zbus_channel_state(RPCDefinitionBase):
    """Query current state of zbus channel"""

    NAME = "zbus_channel_state"
    HELP = "Query current state of zbus channel"
    DESCRIPTION = "Query current state of zbus channel"
    COMMAND_ID = 8

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("channel_id", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("publish_timestamp", ctypes.c_uint64),
            ("publish_count", ctypes.c_uint32),
            ("publish_period_avg_ms", ctypes.c_uint32),
        ]
        vla_field = ("data", 0 * ctypes.c_uint8)
        _pack_ = 1


class application_info(RPCDefinitionBase):
    """Query basic application versions and state"""

    NAME = "application_info"
    HELP = "Query basic application versions and state"
    DESCRIPTION = "Query basic application versions and state"
    COMMAND_ID = 9

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("application_id", ctypes.c_uint32),
            ("version", rpc_struct_mcuboot_img_sem_ver),
            ("network_id", ctypes.c_uint32),
            ("uptime", ctypes.c_uint32),
            ("reboots", ctypes.c_uint32),
            ("kv_crc", ctypes.c_uint32),
            ("data_blocks_internal", ctypes.c_uint32),
            ("data_blocks_external", ctypes.c_uint32),
        ]
        _pack_ = 1


class wifi_scan(RPCDefinitionBase):
    """Scan for WiFi networks"""

    NAME = "wifi_scan"
    HELP = "Scan for WiFi networks"
    DESCRIPTION = "Scan for WiFi networks"
    COMMAND_ID = 10

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("network_count", ctypes.c_uint8),
        ]
        vla_field = ("networks", 0 * rpc_struct_wifi_scan_result)
        _pack_ = 1


class wifi_state(RPCDefinitionBase):
    """Get current WiFi interface state"""

    NAME = "wifi_state"
    HELP = "Get current WiFi interface state"
    DESCRIPTION = "Get current WiFi interface state"
    COMMAND_ID = 11

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("common", rpc_struct_network_state),
            ("wifi", rpc_struct_wifi_state),
        ]
        _pack_ = 1


class last_reboot(RPCDefinitionBase):
    """Retrieve information pertaining to the previous reboot"""

    NAME = "last_reboot"
    HELP = "Retrieve information pertaining to the previous reboot"
    DESCRIPTION = "Retrieve information pertaining to the previous reboot"
    COMMAND_ID = 12

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("reason", ctypes.c_uint8),
            ("epoch_time_source", ctypes.c_uint8),
            ("epoch_time", ctypes.c_uint64),
            ("hardware_flags", ctypes.c_uint32),
            ("uptime", ctypes.c_uint32),
            ("param_1", ctypes.c_uint32),
            ("param_2", ctypes.c_uint32),
            ("thread", 8 * ctypes.c_char),
        ]
        vla_field = ("esf", 0 * ctypes.c_uint32)
        _pack_ = 1


class data_logger_state(RPCDefinitionBase):
    """Get state of a data logger"""

    NAME = "data_logger_state"
    HELP = "Get state of a data logger"
    DESCRIPTION = "Get state of a data logger"
    COMMAND_ID = 13

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("logger", ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("bytes_logged", ctypes.c_uint64),
            ("logical_blocks", ctypes.c_uint32),
            ("physical_blocks", ctypes.c_uint32),
            ("boot_block", ctypes.c_uint32),
            ("current_block", ctypes.c_uint32),
            ("earliest_block", ctypes.c_uint32),
            ("block_size", ctypes.c_uint16),
            ("block_overhead", ctypes.c_uint16),
            ("erase_unit", ctypes.c_uint16),
            ("uptime", ctypes.c_uint32),
        ]
        _pack_ = 1


class data_logger_read(RPCDefinitionBase):
    """Read data from data logger"""

    NAME = "data_logger_read"
    HELP = "Read data from data logger"
    DESCRIPTION = "Read data from data logger"
    COMMAND_ID = 14

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("logger", ctypes.c_uint8),
            ("start_block", ctypes.c_uint32),
            ("last_block", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("sent_len", ctypes.c_uint32),
            ("sent_crc", ctypes.c_uint32),
        ]
        _pack_ = 1


class mem_read(RPCDefinitionBase):
    """Read arbitrary memory (NO ADDRESS VALIDATION PERFORMED)"""

    NAME = "mem_read"
    HELP = "Read arbitrary memory (NO ADDRESS VALIDATION PERFORMED)"
    DESCRIPTION = "Read arbitrary memory (NO ADDRESS VALIDATION PERFORMED)"
    COMMAND_ID = 15

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("address", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("sent_len", ctypes.c_uint32),
            ("sent_crc", ctypes.c_uint32),
        ]
        _pack_ = 1


class infuse_states_query(RPCDefinitionBase):
    """Read current Infuse-IoT application states"""

    NAME = "infuse_states_query"
    HELP = "Read current Infuse-IoT application states"
    DESCRIPTION = "Read current Infuse-IoT application states"
    COMMAND_ID = 16

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("offset", ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("remaining", ctypes.c_uint8),
        ]
        vla_field = ("states", 0 * rpc_struct_infuse_state)
        _pack_ = 1


class infuse_states_update(RPCDefinitionBase):
    """Update Infuse-IoT application states"""

    NAME = "infuse_states_update"
    HELP = "Update Infuse-IoT application states"
    DESCRIPTION = "Update Infuse-IoT application states"
    COMMAND_ID = 17

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("num", ctypes.c_uint8),
        ]
        vla_field = ("states", 0 * rpc_struct_infuse_state)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class data_logger_erase(RPCDefinitionBase):
    """Erase all data from a data logger"""

    NAME = "data_logger_erase"
    HELP = "Erase all data from a data logger"
    DESCRIPTION = "Erase all data from a data logger"
    COMMAND_ID = 18

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("logger", ctypes.c_uint8),
            ("erase_empty", ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class heap_stats(RPCDefinitionBase):
    """Query stats of heaps"""

    NAME = "heap_stats"
    HELP = "Query stats of heaps"
    DESCRIPTION = "Query stats of heaps"
    COMMAND_ID = 19

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("stats", 0 * rpc_struct_heap_info)
        _pack_ = 1


class lte_at_cmd(RPCDefinitionBase):
    """Run AT command against LTE modem"""

    NAME = "lte_at_cmd"
    HELP = "Run AT command against LTE modem"
    DESCRIPTION = "Run AT command against LTE modem"
    COMMAND_ID = 20

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("cmd", 0 * ctypes.c_char)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("rsp", 0 * ctypes.c_char)
        _pack_ = 1


class lte_state(RPCDefinitionBase):
    """Get current LTE interface state"""

    NAME = "lte_state"
    HELP = "Get current LTE interface state"
    DESCRIPTION = "Get current LTE interface state"
    COMMAND_ID = 21

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("common", rpc_struct_network_state),
            ("lte", rpc_struct_lte_state),
        ]
        _pack_ = 1


class data_logger_read_available(RPCDefinitionBase):
    """Read data from data logger, with auto-updating start_block"""

    NAME = "data_logger_read_available"
    HELP = "Read data from data logger, with auto-updating start_block"
    DESCRIPTION = "Read data from data logger, with auto-updating start_block"
    COMMAND_ID = 22

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("logger", ctypes.c_uint8),
            ("start_block", ctypes.c_uint32),
            ("num_blocks", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("sent_len", ctypes.c_uint32),
            ("sent_crc", ctypes.c_uint32),
            ("current_block", ctypes.c_uint32),
            ("start_block_actual", ctypes.c_uint32),
            ("block_size", ctypes.c_uint16),
        ]
        _pack_ = 1


class lte_state_v2(RPCDefinitionBase):
    """Get current LTE interface state (with RAI information)"""

    NAME = "lte_state_v2"
    HELP = "Get current LTE interface state (with RAI information)"
    DESCRIPTION = "Get current LTE interface state (with RAI information)"
    COMMAND_ID = 23

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("common", rpc_struct_network_state),
            ("lte", rpc_struct_lte_state_v2),
        ]
        _pack_ = 1


class coap_download(RPCDefinitionBase):
    """Download a file from a COAP server (Infuse-IoT DTLS protected)"""

    NAME = "coap_download"
    HELP = "Download a file from a COAP server (Infuse-IoT DTLS protected)"
    DESCRIPTION = "Download a file from a COAP server (Infuse-IoT DTLS protected)"
    COMMAND_ID = 30

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("server_address", 48 * ctypes.c_char),
            ("server_port", ctypes.c_uint16),
            ("block_timeout_ms", ctypes.c_uint16),
            ("action", ctypes.c_uint8),
            ("resource_len", ctypes.c_uint32),
            ("resource_crc", ctypes.c_uint32),
        ]
        vla_field = ("resource", 0 * ctypes.c_char)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("resource_len", ctypes.c_uint32),
            ("resource_crc", ctypes.c_uint32),
        ]
        _pack_ = 1


class zperf_upload(RPCDefinitionBase):
    """Network upload bandwidth testing using zperf/iperf"""

    NAME = "zperf_upload"
    HELP = "Network upload bandwidth testing using zperf/iperf"
    DESCRIPTION = "Network upload bandwidth testing using zperf/iperf"
    COMMAND_ID = 31

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer_address", rpc_struct_sockaddr),
            ("sock_type", ctypes.c_uint8),
            ("data_source", ctypes.c_uint8),
            ("duration_ms", ctypes.c_uint32),
            ("rate_kbps", ctypes.c_uint32),
            ("packet_size", ctypes.c_uint16),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("nb_packets_sent", ctypes.c_uint32),
            ("nb_packets_rcvd", ctypes.c_uint32),
            ("nb_packets_lost", ctypes.c_uint32),
            ("nb_packets_outorder", ctypes.c_uint32),
            ("total_len", ctypes.c_uint64),
            ("time_in_us", ctypes.c_uint64),
            ("jitter_in_us", ctypes.c_uint32),
            ("client_time_in_us", ctypes.c_uint64),
            ("packet_size", ctypes.c_uint32),
            ("nb_packets_errors", ctypes.c_uint32),
        ]
        _pack_ = 1


class file_write_basic(RPCDefinitionBase):
    """Write a file to the device"""

    NAME = "file_write_basic"
    HELP = "Write a file to the device"
    DESCRIPTION = "Write a file to the device"
    COMMAND_ID = 40

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("action", ctypes.c_uint8),
            ("file_crc", ctypes.c_uint32),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("recv_len", ctypes.c_uint32),
            ("recv_crc", ctypes.c_uint32),
        ]
        _pack_ = 1


class annotate(RPCDefinitionBase):
    """Write an annotation to the device"""

    NAME = "annotate"
    HELP = "Write an annotation to the device"
    DESCRIPTION = "Write an annotation to the device"
    COMMAND_ID = 41

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("logger", ctypes.c_uint8),
            ("timestamp", ctypes.c_uint32),
        ]
        vla_field = ("annotation", 0 * ctypes.c_char)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class bt_connect_infuse(RPCDefinitionBase):
    """Connect to an Infuse-IoT Bluetooth device"""

    NAME = "bt_connect_infuse"
    HELP = "Connect to an Infuse-IoT Bluetooth device"
    DESCRIPTION = "Connect to an Infuse-IoT Bluetooth device"
    COMMAND_ID = 50

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer", rpc_struct_bt_addr_le),
            ("conn_timeout_ms", ctypes.c_uint16),
            ("subscribe", ctypes.c_uint8),
            ("inactivity_timeout_ms", ctypes.c_uint16),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer", rpc_struct_bt_addr_le),
            ("cloud_public_key", 32 * ctypes.c_uint8),
            ("device_public_key", 32 * ctypes.c_uint8),
            ("network_id", ctypes.c_uint32),
        ]
        _pack_ = 1


class bt_disconnect(RPCDefinitionBase):
    """Disconnect from a Bluetooth device"""

    NAME = "bt_disconnect"
    HELP = "Disconnect from a Bluetooth device"
    DESCRIPTION = "Disconnect from a Bluetooth device"
    COMMAND_ID = 51

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer", rpc_struct_bt_addr_le),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class bt_file_copy_basic(RPCDefinitionBase):
    """Copy a local file to a remote device over Bluetooth"""

    NAME = "bt_file_copy_basic"
    HELP = "Copy a local file to a remote device over Bluetooth"
    DESCRIPTION = "Copy a local file to a remote device over Bluetooth"
    COMMAND_ID = 52

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer", rpc_struct_bt_addr_le),
            ("action", ctypes.c_uint8),
            ("file_idx", ctypes.c_uint8),
            ("file_len", ctypes.c_uint32),
            ("file_crc", ctypes.c_uint32),
            ("ack_period", ctypes.c_uint8),
            ("pipelining", ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class bt_file_copy_coap(RPCDefinitionBase):
    """Copy a file fetched from COAP to a remote device over Bluetooth"""

    NAME = "bt_file_copy_coap"
    HELP = "Copy a file fetched from COAP to a remote device over Bluetooth"
    DESCRIPTION = "Copy a file fetched from COAP to a remote device over Bluetooth"
    COMMAND_ID = 53

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer", rpc_struct_bt_addr_le),
            ("conn_timeout_ms", ctypes.c_uint16),
            ("action", ctypes.c_uint8),
            ("file_idx", ctypes.c_uint8),
            ("ack_period", ctypes.c_uint8),
            ("pipelining", ctypes.c_uint8),
            ("server_address", 48 * ctypes.c_char),
            ("server_port", ctypes.c_uint16),
            ("block_timeout_ms", ctypes.c_uint16),
            ("resource_len", ctypes.c_uint32),
            ("resource_crc", ctypes.c_uint32),
        ]
        vla_field = ("resource", 0 * ctypes.c_char)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("resource_len", ctypes.c_uint32),
            ("resource_crc", ctypes.c_uint32),
        ]
        _pack_ = 1


class bt_mcumgr_reboot(RPCDefinitionBase):
    """Connect to a Bluetooth device and run the MCUMGR reboot command"""

    NAME = "bt_mcumgr_reboot"
    HELP = "Connect to a Bluetooth device and run the MCUMGR reboot command"
    DESCRIPTION = "Connect to a Bluetooth device and run the MCUMGR reboot command"
    COMMAND_ID = 54

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("peer", rpc_struct_bt_addr_le),
            ("conn_timeout_ms", ctypes.c_uint16),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class gravity_reference_update(RPCDefinitionBase):
    """Store the current accelerometer vector as the gravity reference"""

    NAME = "gravity_reference_update"
    HELP = "Store the current accelerometer vector as the gravity reference"
    DESCRIPTION = "Store the current accelerometer vector as the gravity reference"
    COMMAND_ID = 60

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("max_variance", ctypes.c_uint16),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("reference", rpc_struct_xyz_s16),
            ("variance", rpc_struct_xyz_s16),
            ("num_samples", ctypes.c_uint16),
            ("sample_period_us", ctypes.c_uint32),
        ]
        _pack_ = 1


class security_state(RPCDefinitionBase):
    """Query current security state and validate identity"""

    NAME = "security_state"
    HELP = "Query current security state and validate identity"
    DESCRIPTION = "Query current security state and validate identity"
    COMMAND_ID = 30000

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("challenge", 16 * ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("cloud_public_key", 32 * ctypes.c_uint8),
            ("device_public_key", 32 * ctypes.c_uint8),
            ("network_id", ctypes.c_uint32),
            ("challenge_response_type", ctypes.c_uint8),
        ]
        vla_field = ("challenge_response", 0 * ctypes.c_uint8)
        _pack_ = 1


class security_key_update(RPCDefinitionBase):
    """Update key material"""

    NAME = "security_key_update"
    HELP = "Update key material"
    DESCRIPTION = "Update key material"
    COMMAND_ID = 30001

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("key_id", ctypes.c_uint8),
            ("key_action", ctypes.c_uint8),
            ("key_global_identifier", ctypes.c_uint32),
            ("key_bitstream", 32 * ctypes.c_uint8),
            ("reboot_delay", ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class data_sender(RPCDefinitionBase):
    """Send multiple INFUSE_RPC_DATA packets"""

    NAME = "data_sender"
    HELP = "Send multiple INFUSE_RPC_DATA packets"
    DESCRIPTION = "Send multiple INFUSE_RPC_DATA packets"
    COMMAND_ID = 32765

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        _pack_ = 1


class data_receiver(RPCDefinitionBase):
    """Receive multiple INFUSE_RPC_DATA packets"""

    NAME = "data_receiver"
    HELP = "Receive multiple INFUSE_RPC_DATA packets"
    DESCRIPTION = "Receive multiple INFUSE_RPC_DATA packets"
    COMMAND_ID = 32766

    class request(VLACompatLittleEndianStruct):
        _fields_ = [
            ("unaligned_input", ctypes.c_uint8),
        ]
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("recv_len", ctypes.c_uint32),
            ("recv_crc", ctypes.c_uint32),
        ]
        _pack_ = 1


class echo(RPCDefinitionBase):
    """Echo any input data in the response"""

    NAME = "echo"
    HELP = "Echo any input data in the response"
    DESCRIPTION = "Echo any input data in the response"
    COMMAND_ID = 32767

    class request(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("array", 0 * ctypes.c_uint8)
        _pack_ = 1

    class response(VLACompatLittleEndianStruct):
        _fields_ = []
        vla_field = ("array", 0 * ctypes.c_uint8)
        _pack_ = 1


id_type_mapping: dict[int, type[RPCDefinitionBase]] = {
    reboot.COMMAND_ID: reboot,
    fault.COMMAND_ID: fault,
    time_get.COMMAND_ID: time_get,
    time_set.COMMAND_ID: time_set,
    kv_write.COMMAND_ID: kv_write,
    kv_read.COMMAND_ID: kv_read,
    kv_reflect_crcs.COMMAND_ID: kv_reflect_crcs,
    zbus_channel_state.COMMAND_ID: zbus_channel_state,
    application_info.COMMAND_ID: application_info,
    wifi_scan.COMMAND_ID: wifi_scan,
    wifi_state.COMMAND_ID: wifi_state,
    last_reboot.COMMAND_ID: last_reboot,
    data_logger_state.COMMAND_ID: data_logger_state,
    data_logger_read.COMMAND_ID: data_logger_read,
    mem_read.COMMAND_ID: mem_read,
    infuse_states_query.COMMAND_ID: infuse_states_query,
    infuse_states_update.COMMAND_ID: infuse_states_update,
    data_logger_erase.COMMAND_ID: data_logger_erase,
    heap_stats.COMMAND_ID: heap_stats,
    lte_at_cmd.COMMAND_ID: lte_at_cmd,
    lte_state.COMMAND_ID: lte_state,
    data_logger_read_available.COMMAND_ID: data_logger_read_available,
    lte_state_v2.COMMAND_ID: lte_state_v2,
    coap_download.COMMAND_ID: coap_download,
    zperf_upload.COMMAND_ID: zperf_upload,
    file_write_basic.COMMAND_ID: file_write_basic,
    annotate.COMMAND_ID: annotate,
    bt_connect_infuse.COMMAND_ID: bt_connect_infuse,
    bt_disconnect.COMMAND_ID: bt_disconnect,
    bt_file_copy_basic.COMMAND_ID: bt_file_copy_basic,
    bt_file_copy_coap.COMMAND_ID: bt_file_copy_coap,
    bt_mcumgr_reboot.COMMAND_ID: bt_mcumgr_reboot,
    gravity_reference_update.COMMAND_ID: gravity_reference_update,
    security_state.COMMAND_ID: security_state,
    security_key_update.COMMAND_ID: security_key_update,
    data_sender.COMMAND_ID: data_sender,
    data_receiver.COMMAND_ID: data_receiver,
    echo.COMMAND_ID: echo,
}

__all__ = [
    "id_type_mapping",
    "rpc_struct_mcuboot_img_sem_ver",
    "rpc_struct_kv_store_value",
    "rpc_struct_kv_store_crc",
    "rpc_struct_bt_addr_le",
    "rpc_struct_ipv4_address",
    "rpc_struct_ipv6_address",
    "rpc_struct_network_state",
    "rpc_struct_wifi_state",
    "rpc_struct_lte_state",
    "rpc_struct_lte_state_v2",
    "rpc_struct_wifi_scan_result",
    "rpc_struct_xyz_s16",
    "rpc_struct_infuse_state",
    "rpc_struct_sockaddr",
    "rpc_struct_heap_info",
    "rpc_enum_bt_le_addr_type",
    "rpc_enum_file_action",
    "rpc_enum_infuse_bt_characteristic",
    "rpc_enum_data_logger",
    "rpc_enum_zperf_data_source",
    "rpc_enum_key_id",
    "rpc_enum_key_action",
    "rpc_enum_support_status",
    "reboot",
    "fault",
    "time_get",
    "time_set",
    "kv_write",
    "kv_read",
    "kv_reflect_crcs",
    "zbus_channel_state",
    "application_info",
    "wifi_scan",
    "wifi_state",
    "last_reboot",
    "data_logger_state",
    "data_logger_read",
    "mem_read",
    "infuse_states_query",
    "infuse_states_update",
    "data_logger_erase",
    "heap_stats",
    "lte_at_cmd",
    "lte_state",
    "data_logger_read_available",
    "lte_state_v2",
    "coap_download",
    "zperf_upload",
    "file_write_basic",
    "annotate",
    "bt_connect_infuse",
    "bt_disconnect",
    "bt_file_copy_basic",
    "bt_file_copy_coap",
    "bt_mcumgr_reboot",
    "gravity_reference_update",
    "security_state",
    "security_key_update",
    "data_sender",
    "data_receiver",
    "echo",
]



#!/usr/bin/env python3

"""Display received TDFs in a list"""

__author__ = "Jordan Yates"
__copyright__ = "Copyright 2024, Embeint Holdings Pty Ltd"

import time

import tabulate

from infuse_iot.commands import InfuseCommand
from infuse_iot.common import InfuseType
from infuse_iot.generated.tdf_base import TdfStructBase
from infuse_iot.socket_comms import (
    ClientNotificationEpacketReceived,
    LocalClient,
    default_multicast_address,
)
from infuse_iot.tdf import TDF
from infuse_iot.time import InfuseTime


class SubCommand(InfuseCommand):
    NAME = "tdf_list"
    HELP = "Display received TDFs in a list"
    DESCRIPTION = "Display received TDFs in a list"

    def __init__(self, _):
        self._client = LocalClient(default_multicast_address(), 1.0)
        self._decoder = TDF()

    def run(self) -> None:
        while True:
            msg = self._client.receive()
            if msg is None:
                continue
            if not isinstance(msg, ClientNotificationEpacketReceived):
                continue
            if msg.epacket.ptype != InfuseType.TDF:
                continue
            source = msg.epacket.route[0]

            table: list[tuple[str | None, str | None, str, str, str]] = []

            for tdf in self._decoder.decode(msg.epacket.payload):
                t = tdf.data[-1]
                num = len(tdf.data)
                tdf_name: None | str = None
                time_str: None | str = None
                if num > 1:
                    tdf_name = f"{t.NAME}[{num - 1}]"
                else:
                    tdf_name = t.NAME
                if tdf.time is not None:
                    if tdf.period is None:
                        time_str = InfuseTime.utc_time_string(tdf.time)
                    else:
                        offset = (len(tdf.data) - 1) * tdf.period
                        time_str = InfuseTime.utc_time_string(tdf.time + offset)
                else:
                    if tdf.base_idx is not None:
                        time_str = f"IDX {tdf.base_idx}"
                    else:
                        time_str = InfuseTime.utc_time_string(time.time())

                for field in t.iter_fields():
                    if isinstance(field.val, list):
                        # Trailing VLA handling
                        if len(field.val) > 0 and isinstance(field.val[0], TdfStructBase):
                            for idx, val in enumerate(field.val):
                                for subfield in val.iter_fields(f"{field.name}[{idx}]"):
                                    table.append(
                                        (
                                            time_str,
                                            tdf_name,
                                            subfield.name,
                                            subfield.val_fmt(),
                                            subfield.postfix,
                                        )
                                    )
                                    tdf_name = None
                                    time_str = None
                        else:
                            table.append((time_str, tdf_name, f"{field.name}", field.val_fmt(), field.postfix))
                            tdf_name = None
                            time_str = None
                    else:
                        # Standard structs and sub-structs
                        table.append((time_str, tdf_name, field.name, field.val_fmt(), field.postfix))
                        tdf_name = None
                        time_str = None

            print(f"Infuse ID: {source.infuse_id:016x}")
            print(f"Interface: {source.interface.name}")
            print(f"  Address: {source.interface_address}")
            print(f"     RSSI: {source.rssi} dBm")
            print(tabulate.tabulate(table, tablefmt="simple"))



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.device_metadata_update_operation import DeviceMetadataUpdateOperation

if TYPE_CHECKING:
    from ..models.device_metadata import DeviceMetadata


T = TypeVar("T", bound="DeviceMetadataUpdate")


@_attrs_define
class DeviceMetadataUpdate:
    """Metadata update

    Attributes:
        operation (DeviceMetadataUpdateOperation): Operation to perform on metadata, patch to update/add provided
            fields, replace to replace all metadata with provided fields
        value (DeviceMetadata): Metadata fields for device Example: {'Field Name': 'Field Value'}.
    """

    operation: DeviceMetadataUpdateOperation
    value: "DeviceMetadata"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        operation = self.operation.value

        value = self.value.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "operation": operation,
                "value": value,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.device_metadata import DeviceMetadata

        d = dict(src_dict)
        operation = DeviceMetadataUpdateOperation(d.pop("operation"))

        value = DeviceMetadata.from_dict(d.pop("value"))

        device_metadata_update = cls(
            operation=operation,
            value=value,
        )

        device_metadata_update.additional_properties = d
        return device_metadata_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_kv_entry_decoded import DeviceKVEntryDecoded


T = TypeVar("T", bound="DeviceKVEntry")


@_attrs_define
class DeviceKVEntry:
    """
    Attributes:
        key_id (int): Key id
        crc (int): CRC32 of entry value
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        key_name (Union[Unset, str]): Key name - if definition known
        data (Union[Unset, str]): Raw entry data as a base64 encoded string - if not write_only
        decoded (Union[Unset, DeviceKVEntryDecoded]): Decoded entry value - if not write_only and definition known
    """

    key_id: int
    crc: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    key_name: Unset | str = UNSET
    data: Unset | str = UNSET
    decoded: Union[Unset, "DeviceKVEntryDecoded"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        key_id = self.key_id

        crc = self.crc

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        key_name = self.key_name

        data = self.data

        decoded: Unset | dict[str, Any] = UNSET
        if not isinstance(self.decoded, Unset):
            decoded = self.decoded.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "keyId": key_id,
                "crc": crc,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )
        if key_name is not UNSET:
            field_dict["keyName"] = key_name
        if data is not UNSET:
            field_dict["data"] = data
        if decoded is not UNSET:
            field_dict["decoded"] = decoded

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.device_kv_entry_decoded import DeviceKVEntryDecoded

        d = dict(src_dict)
        key_id = d.pop("keyId")

        crc = d.pop("crc")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        key_name = d.pop("keyName", UNSET)

        data = d.pop("data", UNSET)

        _decoded = d.pop("decoded", UNSET)
        decoded: Unset | DeviceKVEntryDecoded
        if isinstance(_decoded, Unset):
            decoded = UNSET
        else:
            decoded = DeviceKVEntryDecoded.from_dict(_decoded)

        device_kv_entry = cls(
            key_id=key_id,
            crc=crc,
            created_at=created_at,
            updated_at=updated_at,
            key_name=key_name,
            data=data,
            decoded=decoded,
        )

        device_kv_entry.additional_properties = d
        return device_kv_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3

import ctypes

import infuse_iot.definitions.rpc as defs
from infuse_iot.commands import InfuseRpcCommand
from infuse_iot.definitions.rpc import rpc_enum_bt_le_addr_type, rpc_enum_file_action, rpc_struct_bt_addr_le
from infuse_iot.rpc_wrappers.coap_download import coap_download, coap_server_file_stats
from infuse_iot.util.argparse import BtLeAddress
from infuse_iot.util.ctypes import bytes_to_uint8
from infuse_iot.zephyr.errno import errno


class bt_file_copy_coap(InfuseRpcCommand, defs.bt_file_copy_coap):
    @classmethod
    def add_parser(cls, parser):
        parser.add_argument(
            "--server",
            type=str,
            default=coap_download.INFUSE_COAP_SERVER_ADDR,
            help="COAP server name",
        )
        parser.add_argument(
            "--port",
            type=int,
            default=coap_download.INFUSE_COAP_SERVER_PORT,
            help="COAP server port",
        )
        parser.add_argument(
            "--resource",
            "-r",
            type=str,
            required=True,
            help="Resource path",
        )
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            "--discard",
            dest="action",
            action="store_const",
            const=rpc_enum_file_action.DISCARD,
            help="Download file and discard without action",
        )
        group.add_argument(
            "--dfu",
            dest="action",
            action="store_const",
            const=rpc_enum_file_action.APP_IMG,
            help="Download complete image file and perform DFU",
        )
        group.add_argument(
            "--cpatch",
            dest="action",
            action="store_const",
            const=rpc_enum_file_action.APP_CPATCH,
            help="Download CPatch binary diff and perform DFU",
        )
        group.add_argument(
            "--nrf91-modem",
            dest="action",
            action="store_const",
            const=rpc_enum_file_action.NRF91_MODEM_DIFF,
            help="nRF91 LTE modem diff upgrade",
        )
        addr_group = parser.add_mutually_exclusive_group(required=True)
        addr_group.add_argument("--public", type=BtLeAddress, help="Public Bluetooth address")
        addr_group.add_argument("--random", type=BtLeAddress, help="Random Bluetooth address")
        parser.add_argument("--conn-timeout", type=int, default=5000, help="Connection timeout (ms)")
        parser.add_argument("--bt-pipelining", type=int, default=4, help="Bluetooth data pipelining")

    def __init__(self, args):
        self.server = args.server.encode("utf-8")
        self.port = args.port
        self.resource = args.resource.encode("utf-8")
        self.action = args.action
        self.conn_timeout = args.conn_timeout
        self.pipelining = args.bt_pipelining
        if args.public:
            self.peer = rpc_struct_bt_addr_le(
                rpc_enum_bt_le_addr_type.PUBLIC,
                bytes_to_uint8(args.public.to_bytes(6, "little")),
            )
        else:
            self.peer = rpc_struct_bt_addr_le(
                rpc_enum_bt_le_addr_type.RANDOM,
                bytes_to_uint8(args.random.to_bytes(6, "little")),
            )
        self.file_len, self.file_crc = coap_server_file_stats(args.server, args.resource)

    def request_struct(self):
        class request(ctypes.LittleEndianStructure):
            _fields_ = [
                *self.request._fields_,
                ("resource", (len(self.resource) + 1) * ctypes.c_char),
            ]
            _pack_ = 1

        return request(
            self.peer,
            self.conn_timeout,
            self.action,
            0,
            1,
            self.pipelining,
            self.server,
            self.port,
            2000,
            self.file_len,
            self.file_crc,
            self.resource,
        )

    def handle_response(self, return_code, response):
        if return_code != 0:
            print(f"Failed to download file ({errno.strerror(-return_code)})")
            return
        else:
            print("File downloaded and copied")
            print(f"\tLength: {response.resource_len}")
            print(f"\t   CRC: 0x{response.resource_crc:08x}")



#!/usr/bin/env python3

import enum


class RegistrationState(enum.IntEnum):
    """Network registration state (3GPP TS 127.007)"""

    NOT_REGISTERED = 0
    REGISTERED_HOME = 1
    SEARCHING = 2
    REGISTRATION_DENIED = 3
    REGISTRATION_UNKNOWN = 4
    REGISTERED_ROAMING = 5
    UICC_FAIL = 90

    def __str__(self):
        pretty_names = {
            self.NOT_REGISTERED: "Not registered",
            self.REGISTERED_HOME: "Registered (Home Network)",
            self.SEARCHING: "Searching",
            self.REGISTRATION_DENIED: "Registration denied",
            self.REGISTRATION_UNKNOWN: "Unknown",
            self.REGISTERED_ROAMING: "Registered (Roaming)",
            self.UICC_FAIL: "SIM card failure",
        }
        return pretty_names[self]


class AccessTechnology(enum.IntEnum):
    """Access Technology (3GPP TS 127.007)"""

    GSM = 0
    GSM_COMPACT = 1
    UTRAN = 2
    GSM_EGPRS = 3
    UTRAN_HSDPA = 4
    UTRAN_HSUPA = 5
    UTRAN_HSDPA_HSUPA = 6
    E_UTRAN = 7
    EC_GSM_IOT = 8
    E_UTRAN_NB_S1 = 9
    E_UTRA_5G_CN = 10
    NR_5G_CN = 11
    NG_RAN = 12
    E_UTRA_NR_DUAL = 13
    E_UTRAN_NB_S1_SAT = 14
    E_UTRAN_WB_S1_SAT = 15
    NG_RAN_SAT = 16
    UNKNOWN = 255

    def __str__(self):
        pretty_names = {
            self.GSM: "GSM (2G, 3GPP Rel 99)",
            self.GSM_COMPACT: "GSM Compact (2G, 3GPP Rel 99)",
            self.UTRAN: "UTRAN (3G, 3GPP Rel 99)",
            self.GSM_EGPRS: "GSM Enhanced (2.5G, 3GPP Rel 99)",
            self.UTRAN_HSDPA: "UTRAN High Speed Downlink (3.5G, 3GPP Rel 5)",
            self.UTRAN_HSUPA: "UTRAN High Speed Uplink (3.75G, 3GPP Rel 6)",
            self.UTRAN_HSDPA_HSUPA: "UTRAN High Speed Uplink/Downlink (3.75G, 3GPP Rel 6)",
            self.E_UTRAN: "LTE/Evolved UTRAN (4G, 3GPP Rel 8)",
            self.EC_GSM_IOT: "Extended Coverage GSM for IoT (2G, 3GPP Rel 13)",
            self.E_UTRAN_NB_S1: "EUTRAN Narrowband-IoT (4G, 3GPP Rel 13)",
            self.E_UTRA_5G_CN: "LTE/E-UTRA connected to 5G Core Network (5G, 3GPP Rel 15)",
            self.NR_5G_CN: "New Radio with 5G Core Network (5G, 3GPP Rel 15)",
            self.NG_RAN: "Next Generation RAN (5G, 3GPP Rel 15)",
            self.E_UTRA_NR_DUAL: "LTE/E-UTRA & NR dual connectivity (5G, 3GPP Rel 15)",
            self.E_UTRAN_NB_S1_SAT: "Narrowband-IoT over Satellite (4G, 3GPP Rel 17)",
            self.E_UTRAN_WB_S1_SAT: "LTE (wideband) over Satellite (4G, 3GPP Rel 17)",
            self.NG_RAN_SAT: "Next Generation RAN over Satellite (5G, 3GPP Rel 17)",
            self.UNKNOWN: "Unknown",
        }
        return pretty_names[self]


class LteSystemMode(enum.IntEnum):
    LTE_M = 1
    NB_IOT = 2
    GNSS = 3
    LTE_M_GNSS = 4
    NB_IOT_GNSS = 5
    LTE_M_NB_IOT = 6
    LTE_M_NB_IOT_GNSS = 7
    DEFAULT = 0xFF

    def __str__(self):
        pretty_names = {
            self.LTE_M: "LTE-M",
            self.NB_IOT: "NB-IoT",
            self.GNSS: "GNSS",
            self.LTE_M_GNSS: "LTE-M + GNSS",
            self.NB_IOT_GNSS: "NB-IoT + GNSS",
            self.LTE_M_NB_IOT: "LTE-M + NB-IoT",
            self.LTE_M_NB_IOT_GNSS: "LTE-M + NB-IoT + GNSS",
            self.DEFAULT: "Modem default",
        }
        return pretty_names[self]


class LteSystemPreference(enum.IntEnum):
    AUTO = 0
    LTE_M = 1
    NB_IOT = 2
    PLMN_LTE_M = 3
    PLMN_NB_IOT = 4

    def __str__(self):
        pretty_names = {
            self.AUTO: "No preference",
            self.LTE_M: "LTE-M",
            self.NB_IOT: "NB-IoT",
            self.PLMN_LTE_M: "PLMN > LTE-M",
            self.PLMN_NB_IOT: "PLMN > NB-IoT",
        }
        return pretty_names[self]


class LteBand:
    def __init__(self, band, freq_dl_low, offset_dl, freq_ul_low, offset_ul):
        self.band = band
        self.fdl_low = freq_dl_low
        self.ndl = offset_dl
        self.ful_low = freq_ul_low
        self.nul = offset_ul


class LteBands:
    bands = {
        1: LteBand(1, 2110, 0, 1920, 18000),
        2: LteBand(2, 1930, 600, 1850, 18600),
        3: LteBand(3, 1805, 1200, 1710, 19200),
        4: LteBand(4, 2110, 1950, 1710, 19950),
        5: LteBand(5, 869, 2400, 824, 20400),
        6: LteBand(6, 875, 2650, 830, 20650),
        7: LteBand(7, 2620, 2750, 2500, 20750),
        8: LteBand(8, 925, 3450, 880, 21450),
        9: LteBand(9, 1844.9, 3800, 1749.9, 21800),
        10: LteBand(10, 2110, 4150, 1710, 22150),
        11: LteBand(11, 1475.9, 4750, 1427.9, 22750),
        12: LteBand(12, 729, 5010, 699, 23010),
        13: LteBand(13, 746, 5180, 777, 23180),
        14: LteBand(14, 758, 5280, 788, 23280),
        17: LteBand(17, 734, 5730, 704, 23730),
        18: LteBand(18, 860, 5850, 815, 23850),
        19: LteBand(19, 875, 6000, 830, 24000),
        20: LteBand(20, 791, 6150, 832, 24150),
        21: LteBand(21, 1495.9, 6450, 1447.9, 24450),
        22: LteBand(22, 3510, 6600, 3410, 24600),
        23: LteBand(23, 2180, 7500, 2000, 25500),
        24: LteBand(24, 1525, 7700, 1626.5, 25700),
        25: LteBand(25, 1930, 8040, 1850, 26040),
        26: LteBand(26, 859, 8690, 814, 26690),
        27: LteBand(27, 852, 9040, 807, 27040),
        28: LteBand(28, 758, 9210, 703, 27210),
        31: LteBand(31, 462.5, 9870, 452.5, 27760),
        33: LteBand(33, 1900, 36000, 1900, 36000),
        34: LteBand(34, 2010, 36200, 2010, 36200),
        35: LteBand(35, 1850, 36350, 1850, 36350),
        36: LteBand(36, 1930, 36950, 1930, 36950),
        37: LteBand(37, 1910, 37550, 1910, 37550),
        38: LteBand(38, 2570, 37750, 2570, 37750),
        39: LteBand(39, 1880, 38250, 1880, 38250),
        40: LteBand(40, 2300, 38650, 2300, 38650),
        41: LteBand(41, 2496, 39650, 2496, 39650),
        42: LteBand(42, 3400, 41590, 3400, 41590),
        43: LteBand(43, 3600, 43590, 3600, 43590),
        44: LteBand(44, 703, 45590, 703, 45590),
    }

    @classmethod
    def earfcn_to_freq(cls, earfcn_dl):
        prev = None
        for b in cls.bands.values():
            if prev is None:
                prev = b
                continue
            if earfcn_dl >= prev.ndl and earfcn_dl < b.ndl:
                info = prev

                ul_offset = info.nul - info.ndl
                earfcn_ul = earfcn_dl + ul_offset

                freq_dl = info.fdl_low + 0.1 * (earfcn_dl - info.ndl)
                freq_ul = info.ful_low + 0.1 * (earfcn_ul - info.nul)
                return (freq_dl, freq_ul)

            prev = b
        raise ValueError(f"EARFCN {earfcn_dl} is invalid")


class MobileCountryCodes:
    _names = {
        289: "Abkhazia",
        412: "Afghanistan",
        276: "Albania",
        603: "Algeria",
        544: "American Samoa (United States of America)",
        213: "Andorra",
        631: "Angola",
        365: "Anguilla (United Kingdom)",
        344: "Antigua and Barbuda",
        722: "Argentina",
        283: "Armenia",
        363: "Aruba",
        505: "Australia",
        232: "Austria",
        400: "Azerbaijan",
        364: "Bahamas",
        426: "Bahrain",
        470: "Bangladesh",
        342: "Barbados",
        257: "Belarus",
        206: "Belgium",
        702: "Belize",
        616: "Benin",
        350: "Bermuda",
        402: "Bhutan",
        736: "Bolivia",
        362: "Bonaire, Sint Eustatius and Saba, Curaçao, Saint Maarten",
        218: "Bosnia and Herzegovina",
        652: "Botswana",
        724: "Brazil",
        995: "British Indian Ocean Territory (United Kingdom)",
        348: "British Virgin Islands (United Kingdom)",
        528: "Brunei",
        284: "Bulgaria",
        613: "Burkina Faso",
        642: "Burundi",
        456: "Cambodia",
        624: "Cameroon",
        302: "Canada",
        625: "Cape Verde",
        346: "Cayman Islands (United Kingdom)",
        623: "Central African Republic",
        622: "Chad",
        730: "Chile",
        460: "China",
        461: "China",
        732: "Colombia",
        654: "Comoros",
        629: "Congo",
        548: "Cook Islands (Pacific Ocean)",
        712: "Costa Rica",
        219: "Croatia",
        368: "Cuba",
        280: "Cyprus",
        230: "Czech Republic",
        630: "Democratic Republic of the Congo",
        238: "Denmark (Kingdom of Denmark)",
        638: "Djibouti",
        366: "Dominica",
        370: "Dominican Republic",
        514: "East Timor",
        740: "Ecuador",
        602: "Egypt",
        706: "El Salvador",
        627: "Equatorial Guinea",
        657: "Eritrea",
        248: "Estonia",
        636: "Ethiopia",
        750: "Falkland Islands (United Kingdom)",
        288: "Faroe Islands (Kingdom of Denmark)",
        542: "Fiji",
        244: "Finland",
        208: "France",
        742: "French Guiana (France)",
        647: "French Indian Ocean Territories (France)",
        547: "French Polynesia (France)",
        628: "Gabon",
        607: "Gambia",
        282: "Georgia",
        262: "Germany",
        620: "Ghana",
        266: "Gibraltar (United Kingdom)",
        202: "Greece",
        290: "Greenland (Kingdom of Denmark)",
        352: "Grenada",
        340: "Guadeloupe, Martinique, Saint Barthélemy, Saint Martin (France)",
        704: "Guatemala",
        611: "Guinea",
        632: "Guinea-Bissau",
        738: "Guyana",
        372: "Haiti",
        708: "Honduras",
        454: "Hong Kong",
        216: "Hungary",
        274: "Iceland",
        404: "India",
        405: "India",
        406: "India",
        510: "Indonesia",
        432: "Iran",
        418: "Iraq",
        272: "Ireland",
        425: "Israel, Palestine",
        222: "Italy",
        612: "Ivory Coast",
        338: "Jamaica",
        440: "Japan",
        441: "Japan",
        416: "Jordan",
        401: "Kazakhstan",
        639: "Kenya",
        545: "Kiribati",
        467: "rea",
        450: "rea",
        221: "Kosovo",
        419: "Kuwait",
        437: "Kyrgyzstan",
        457: "Laos",
        247: "Latvia",
        415: "Lebanon",
        651: "Lesotho",
        618: "Liberia",
        606: "Libya",
        295: "Liechtenstein",
        246: "Lithuania",
        270: "Luxembourg",
        455: "Macau",
        294: "North Macedonia",
        646: "Madagascar",
        650: "Malawi",
        502: "Malaysia",
        472: "Maldives",
        610: "Mali",
        278: "Malta",
        551: "Marshall Islands",
        609: "Mauritania",
        617: "Mauritius",
        334: "Mexico",
        550: "cronesia",
        259: "Moldova",
        212: "Monaco",
        428: "Mongolia",
        297: "Montenegro",
        354: "Montserrat (United Kingdom)",
        604: "Morocco",
        643: "Mozambique",
        414: "Myanmar",
        649: "Namibia",
        536: "Nauru",
        429: "Nepal",
        204: "Netherlands (Kingdom of the Netherlands)",
        546: "New Caledonia",
        530: "New Zealand",
        710: "Nicaragua",
        614: "Niger",
        621: "Nigeria",
        555: "Niue",
        242: "Norway",
        422: "Oman",
        410: "Pakistan",
        552: "Palau",
        714: "Panama",
        537: "Papua New Guinea",
        744: "Paraguay",
        716: "Peru",
        515: "Philippines",
        260: "Poland",
        268: "Portugal",
        330: "Puerto Rico (United States of America)",
        427: "Qatar",
        226: "Romania",
        250: "Russian Federation",
        635: "Rwanda",
        658: "int Helena",
        356: "Saint Kitts and Nevis",
        358: "Saint Lucia",
        308: "Saint Pierre and Miquelon",
        360: "Saint Vincent and the Grenadines",
        549: "Samoa",
        292: "San Marino",
        626: "São Tomé and Príncipe",
        420: "Saudi Arabia",
        608: "Senegal",
        220: "Serbia",
        633: "Seychelles",
        619: "Sierra Leone",
        525: "Singapore",
        231: "Slovakia",
        293: "Slovenia",
        540: "Solomon Islands",
        637: "Somalia",
        655: "South Africa",
        659: "South Sudan",
        214: "Spain",
        413: "Sri Lanka",
        634: "Sudan",
        746: "Suriname",
        653: "Swaziland",
        240: "Sweden",
        228: "Switzerland",
        417: "Syria",
        466: "Taiwan",
        436: "Tajikistan",
        640: "Tanzania",
        520: "Thailand",
        615: "Togo",
        554: "Tokelau",
        539: "Tonga",
        374: "Trinidad and Tobago",
        605: "Tunisia",
        286: "Turkey",
        438: "Turkmenistan",
        376: "Turks and Caicos Islands",
        553: "Tuvalu",
        641: "Uganda",
        255: "Ukraine",
        424: "United Arab Emirates",
        430: "United Arab Emirates (Abu Dhabi)",
        431: "United Arab Emirates (Dubai)",
        234: "United Kingdom",
        235: "United Kingdom",
        310: "United States of America",
        311: "United States of America",
        312: "United States of America",
        313: "United States of America",
        314: "United States of America",
        315: "United States of America",
        316: "United States of America",
        332: "United States Virgin Islands",
        748: "Uruguay",
        434: "Uzbekistan",
        541: "Vanuatu",
        734: "Venezuela",
        452: "Vietnam",
        543: "Wallis and Futuna",
        421: "Yemen",
        645: "Zambia",
        648: "Zimbabwe",
    }

    @classmethod
    def name_from_mcc(cls, mcc):
        return cls._names.get(mcc, "Unknown")



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.definitions_field_definition import DefinitionsFieldDefinition


T = TypeVar("T", bound="DefinitionsKVDefinition")


@_attrs_define
class DefinitionsKVDefinition:
    """
    Attributes:
        name (str):
        description (str):
        fields (list['DefinitionsFieldDefinition']):
        reflect (Union[Unset, bool]):
        read_only (Union[Unset, bool]):
        write_only (Union[Unset, bool]):
        default (Union[Unset, str]):
        depends_on (Union[Unset, str]):
        range_ (Union[Unset, int]):
    """

    name: str
    description: str
    fields: list["DefinitionsFieldDefinition"]
    reflect: Unset | bool = UNSET
    read_only: Unset | bool = UNSET
    write_only: Unset | bool = UNSET
    default: Unset | str = UNSET
    depends_on: Unset | str = UNSET
    range_: Unset | int = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description = self.description

        fields = []
        for fields_item_data in self.fields:
            fields_item = fields_item_data.to_dict()
            fields.append(fields_item)

        reflect = self.reflect

        read_only = self.read_only

        write_only = self.write_only

        default = self.default

        depends_on = self.depends_on

        range_ = self.range_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "description": description,
                "fields": fields,
            }
        )
        if reflect is not UNSET:
            field_dict["reflect"] = reflect
        if read_only is not UNSET:
            field_dict["read_only"] = read_only
        if write_only is not UNSET:
            field_dict["write_only"] = write_only
        if default is not UNSET:
            field_dict["default"] = default
        if depends_on is not UNSET:
            field_dict["depends_on"] = depends_on
        if range_ is not UNSET:
            field_dict["range"] = range_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.definitions_field_definition import DefinitionsFieldDefinition

        d = dict(src_dict)
        name = d.pop("name")

        description = d.pop("description")

        fields = []
        _fields = d.pop("fields")
        for fields_item_data in _fields:
            fields_item = DefinitionsFieldDefinition.from_dict(fields_item_data)

            fields.append(fields_item)

        reflect = d.pop("reflect", UNSET)

        read_only = d.pop("read_only", UNSET)

        write_only = d.pop("write_only", UNSET)

        default = d.pop("default", UNSET)

        depends_on = d.pop("depends_on", UNSET)

        range_ = d.pop("range", UNSET)

        definitions_kv_definition = cls(
            name=name,
            description=description,
            fields=fields,
            reflect=reflect,
            read_only=read_only,
            write_only=write_only,
            default=default,
            depends_on=depends_on,
            range_=range_,
        )

        definitions_kv_definition.additional_properties = d
        return definitions_kv_definition

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3

import ctypes
from typing import Any

from tabulate import tabulate

import infuse_iot.definitions.kv as kv
import infuse_iot.definitions.rpc as defs
from infuse_iot.commands import InfuseRpcCommand
from infuse_iot.zephyr.errno import errno


class kv_read(InfuseRpcCommand, defs.kv_read):
    @classmethod
    def add_parser(cls, parser):
        parser.add_argument("--keys", "-k", required=True, type=int, nargs="+", help="Keys to read")

    def __init__(self, args):
        self.keys = args.keys

    def request_struct(self):
        keys = (ctypes.c_uint16 * len(self.keys))(*self.keys)
        return bytes(self.request(len(self.keys))) + bytes(keys)

    def request_json(self):
        return {"num": str(len(self.keys)), "keys": [str(k) for k in self.keys]}

    def handle_response(self, return_code, response):
        if return_code != 0:
            print(f"Invalid data buffer ({errno.strerror(-return_code)})")
            return

        for r in response.values:
            if r.len > 0:
                b = bytes(r.data)
                try:
                    kv_type = kv.slots.ID_MAPPING[r.id]
                except KeyError:
                    print(f"Key: {r.id} ({r.len} bytes):")
                    print(f"\tHex: {b.hex()}")
                    continue
                kv_val = kv_type.vla_from_buffer_copy(b)

                print(f"Key: {kv_type.NAME} ({r.len} bytes):")
                print(f"\tHex: {b.hex()}")

                fields = []
                for field_name, field_val in kv_val.iter_fields():
                    fmt_val: Any
                    if isinstance(field_val, ctypes.Array):
                        if field_val._type_ == ctypes.c_char:
                            fmt_val = bytes(field_val).decode("utf-8")
                        elif field_val._type_ == ctypes.c_ubyte:
                            fmt_val = bytes(field_val).hex()
                        else:
                            fmt_val = list(field_val)
                    else:
                        fmt_val = field_val
                    fields.append((field_name, fmt_val))
                print(tabulate(fields))
            else:
                print(f"Key: {r.id} (Failed to read '{errno.strerror(-r.len)}')")



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_metadata import DeviceMetadata
    from ..models.device_state import DeviceState
    from ..models.new_device_state import NewDeviceState


T = TypeVar("T", bound="DeviceAndState")


@_attrs_define
class DeviceAndState:
    """
    Attributes:
        id (UUID): Generated UUID for organisation
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        mcu_id (str): Device's MCU ID as a hex string Example: 0011223344556677.
        board_id (UUID): ID of board of device
        organisation_id (UUID): ID of organisation for board to exist in
        state (DeviceState):
        device_id (Union[Unset, str]): 8 byte DeviceID as a hex string (if not provided will be auto-generated) Example:
            d291d4d66bf0a955.
        metadata (Union[Unset, DeviceMetadata]): Metadata fields for device Example: {'Field Name': 'Field Value'}.
        initial_device_state (Union[Unset, NewDeviceState]):
    """

    id: UUID
    created_at: datetime.datetime
    updated_at: datetime.datetime
    mcu_id: str
    board_id: UUID
    organisation_id: UUID
    state: "DeviceState"
    device_id: Unset | str = UNSET
    metadata: Union[Unset, "DeviceMetadata"] = UNSET
    initial_device_state: Union[Unset, "NewDeviceState"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = str(self.id)

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        mcu_id = self.mcu_id

        board_id = str(self.board_id)

        organisation_id = str(self.organisation_id)

        state = self.state.to_dict()

        device_id = self.device_id

        metadata: Unset | dict[str, Any] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        initial_device_state: Unset | dict[str, Any] = UNSET
        if not isinstance(self.initial_device_state, Unset):
            initial_device_state = self.initial_device_state.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "mcuId": mcu_id,
                "boardId": board_id,
                "organisationId": organisation_id,
                "state": state,
            }
        )
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if initial_device_state is not UNSET:
            field_dict["initialDeviceState"] = initial_device_state

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.device_metadata import DeviceMetadata
        from ..models.device_state import DeviceState
        from ..models.new_device_state import NewDeviceState

        d = dict(src_dict)
        id = UUID(d.pop("id"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        mcu_id = d.pop("mcuId")

        board_id = UUID(d.pop("boardId"))

        organisation_id = UUID(d.pop("organisationId"))

        state = DeviceState.from_dict(d.pop("state"))

        device_id = d.pop("deviceId", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Unset | DeviceMetadata
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = DeviceMetadata.from_dict(_metadata)

        _initial_device_state = d.pop("initialDeviceState", UNSET)
        initial_device_state: Unset | NewDeviceState
        if isinstance(_initial_device_state, Unset):
            initial_device_state = UNSET
        else:
            initial_device_state = NewDeviceState.from_dict(_initial_device_state)

        device_and_state = cls(
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            mcu_id=mcu_id,
            board_id=board_id,
            organisation_id=organisation_id,
            state=state,
            device_id=device_id,
            metadata=metadata,
            initial_device_state=initial_device_state,
        )

        device_and_state.additional_properties = d
        return device_and_state

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3

"""Connect to remote Bluetooth device serial logs"""

__author__ = "Jordan Yates"
__copyright__ = "Copyright 2024, Embeint Holdings Pty Ltd"


from infuse_iot.commands import InfuseCommand
from infuse_iot.common import InfuseType
from infuse_iot.epacket import interface
from infuse_iot.socket_comms import (
    ClientNotificationConnectionDropped,
    ClientNotificationEpacketReceived,
    GatewayRequestConnectionRequest,
    LocalClient,
    default_multicast_address,
)
from infuse_iot.tdf import TDF


class SubCommand(InfuseCommand):
    NAME = "bt_log"
    HELP = "Connect to remote Bluetooth device serial logs"
    DESCRIPTION = "Connect to remote Bluetooth device serial logs"

    def __init__(self, args):
        self._client = LocalClient(default_multicast_address(), 1.0)
        self._decoder = TDF()
        self._id = args.id
        self._data = args.data
        self._conn_timeout = args.conn_timeout

    @classmethod
    def add_parser(cls, parser):
        parser.add_argument("--id", type=lambda x: int(x, 0), help="Infuse ID to receive logs for")
        parser.add_argument("--data", action="store_true", help="Subscribe to the data characteristic as well")
        parser.add_argument(
            "--conn-timeout", type=int, default=10000, help="Timeout to wait for a connection to the device (ms)"
        )

    def run(self):
        try:
            types = GatewayRequestConnectionRequest.DataType.LOGGING
            if self._data:
                types |= GatewayRequestConnectionRequest.DataType.DATA
            with self._client.connection(self._id, types, self._conn_timeout) as _:
                while True:
                    evt = self._client.receive()
                    if evt is None:
                        continue
                    if isinstance(evt, ClientNotificationConnectionDropped):
                        print(f"Connection to {self._id:016x} lost")
                        break
                    if not isinstance(evt, ClientNotificationEpacketReceived):
                        continue
                    source = evt.epacket.route[0]
                    if source.infuse_id != self._id:
                        continue
                    if source.interface != interface.ID.BT_CENTRAL:
                        continue

                    if evt.epacket.ptype == InfuseType.SERIAL_LOG:
                        print(evt.epacket.payload.decode("utf-8"), end="")
                    if evt.epacket.ptype == InfuseType.TDF:
                        for tdf in self._decoder.decode(evt.epacket.payload):
                            t = tdf.data[-1]
                            t_str = f"{tdf.time:.3f}" if tdf.time else "N/A"
                            if len(tdf.data) > 1:
                                print(f"{t_str} TDF: {t.NAME}[{len(tdf.data)}]")
                            else:
                                print(f"{t_str} TDF: {t.NAME}")

        except KeyboardInterrupt:
            print(f"Disconnecting from {self._id:016x}")
        except ConnectionRefusedError:
            print(f"Unable to connect to {self._id:016x}")



#!/usr/bin/env python3

import binascii
import time

import infuse_iot.definitions.rpc as defs
from infuse_iot.commands import InfuseRpcCommand
from infuse_iot.util.ctypes import UINT32_MAX
from infuse_iot.zephyr.errno import errno


class data_logger_read(InfuseRpcCommand, defs.data_logger_read):
    RPC_DATA_RECEIVE = True

    @classmethod
    def add_parser(cls, parser):
        logger = parser.add_mutually_exclusive_group(required=True)
        logger.add_argument("--onboard", action="store_true", help="Onboard flash logger")
        logger.add_argument("--removable", action="store_true", help="Removable flash logger (SD)")
        parser.add_argument("--start", type=int, default=0, help="First logger block to read (default 0)")
        parser.add_argument("--last", type=int, default=UINT32_MAX, help="Last logger block to read (default all)")

    def __init__(self, args):
        self.infuse_id = args.id
        self.start = args.start
        self.last = args.last
        if args.onboard:
            self.logger = defs.rpc_enum_data_logger.FLASH_ONBOARD
        elif args.removable:
            self.logger = defs.rpc_enum_data_logger.FLASH_REMOVABLE
        else:
            raise NotImplementedError
        self.expected_offset = 0
        self.output = bytearray()
        self.start_time = time.time()

    def request_struct(self):
        return self.request(self.logger, self.start, self.last)

    def request_json(self):
        return {"logger": self.logger.name, "start_block": self.start, "last_block": self.last}

    def data_recv_cb(self, offset: int, data: bytes) -> None:
        if self.expected_offset == 0:
            self.start_time = time.time()
        if offset == self.expected_offset:
            self.output += data
            # Next expected offset
            self.expected_offset = offset + len(data)
        else:
            missing = offset - self.expected_offset
            if missing > 0:
                print(f"Missed {missing:d} bytes from offset 0x{self.expected_offset:08x}")
                self.output += b"\x00" * missing
                self.output += data
                self.expected_offset = offset + len(data)
            else:
                print(f"Received missing bytes from offset 0x{self.expected_offset:08x}")
                self.output[offset : offset + len(data)] = data

    def handle_response(self, return_code, response):
        end_time = time.time()
        if return_code != 0:
            print(f"Failed to read data logger ({errno.strerror(-return_code)})")
            return

        if response.sent_len != len(self.output):
            print(f"Unexpected received length ({response.sent_len} != {len(self.output)})")

        if response.sent_crc != binascii.crc32(self.output):
            print(f"Unexpected received CRC ({response.sent_crc:08x} != {binascii.crc32(self.output):08x})")

        duration = end_time - self.start_time
        bitrate = (len(self.output) * 8) / duration / 1024

        file_prefix = f"{self.infuse_id:016x}" if self.infuse_id else "gateway"
        output_file = f"{file_prefix}_{self.logger.name}.bin"
        with open(output_file, "wb") as f:
            f.write(self.output)
        print(f"Wrote {response.sent_len:d} bytes to {output_file} in {duration:.2f} sec ({bitrate:.3f} kbps)")



#!/usr/bin/env python3

import ctypes
import enum

from typing_extensions import Self

import infuse_iot.definitions.rpc as rpc_defs
import infuse_iot.definitions.tdf as tdf_defs
from infuse_iot.epacket.common import Serializable
from infuse_iot.util.ctypes import bytes_to_uint8


class ID(enum.Enum):
    """Interface identifier"""

    SERIAL = 0
    UDP = 1
    BT_ADV = 2
    BT_PERIPHERAL = 3
    BT_CENTRAL = 4


class Address(Serializable):
    class SerialAddr(Serializable):
        def __str__(self):
            return ""

        def len(self):
            return 0

        def to_json(self) -> dict:
            return {"i": "SERIAL"}

        @classmethod
        def from_json(cls, values: dict) -> Self:
            return cls()

    class BluetoothLeAddr(Serializable):
        class CtypesFormat(ctypes.LittleEndianStructure):
            _fields_ = [
                ("type", ctypes.c_uint8),
                ("addr", 6 * ctypes.c_uint8),
            ]
            _pack_ = 1

        def __init__(self, addr_type: int, addr_val: int):
            self.addr_type = addr_type
            self.addr_val = addr_val

        def __hash__(self) -> int:
            return (self.addr_type << 48) + self.addr_val

        def __eq__(self, another) -> bool:
            return self.addr_type == another.addr_type and self.addr_val == another.addr_val

        def __str__(self) -> str:
            t = "random" if self.addr_type == 1 else "public"
            v = ":".join([f"{x:02x}" for x in self.addr_val.to_bytes(6, "big")])
            return f"{v} ({t})"

        def len(self):
            return ctypes.sizeof(self.CtypesFormat)

        def to_ctype(self) -> CtypesFormat:
            """Convert the address to the ctype format"""
            return self.CtypesFormat(self.addr_type, bytes_to_uint8(self.addr_val.to_bytes(6, "little")))

        def to_json(self) -> dict:
            return {"i": "BT", "t": self.addr_type, "v": self.addr_val}

        @classmethod
        def from_json(cls, values: dict) -> Self:
            return cls(values["t"], values["v"])

        def to_rpc_struct(self) -> rpc_defs.rpc_struct_bt_addr_le:
            """Convert the address to the common RPC address structure"""

            return rpc_defs.rpc_struct_bt_addr_le(self.addr_type, bytes_to_uint8(self.addr_val.to_bytes(6, "little")))

        @classmethod
        def from_rpc_struct(cls, struct: rpc_defs.rpc_struct_bt_addr_le):
            """Create instance from the common RPC address structure"""

            return cls(struct.type, int.from_bytes(struct.val, "little"))

        @classmethod
        def from_tdf_struct(cls, struct: tdf_defs.structs.tdf_struct_bt_addr_le):
            """Create instance from the common TDF address structure"""

            return cls(struct.type, struct.val)

    def __init__(self, val):
        self.val = val

    def __str__(self):
        return str(self.val)

    def len(self):
        return self.val.len()

    def to_json(self) -> dict:
        return self.val.to_json()

    @classmethod
    def from_json(cls, values: dict) -> Self:
        if values["i"] == "BT":
            return cls(cls.BluetoothLeAddr.from_json(values))
        elif values["i"] == "SERIAL":
            return cls(cls.SerialAddr())
        raise NotImplementedError("Unknown address type")

    @classmethod
    def from_bytes(cls, interface: ID, stream: bytes) -> Self:
        assert interface in [
            ID.BT_ADV,
            ID.BT_PERIPHERAL,
            ID.BT_CENTRAL,
        ]

        c = cls.BluetoothLeAddr.CtypesFormat.from_buffer_copy(stream)
        return cls(cls.BluetoothLeAddr(c.type, int.from_bytes(bytes(c.addr), "little")))



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.definitions_rpc_command_default_auth import DefinitionsRPCCommandDefaultAuth
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.definitions_field_definition import DefinitionsFieldDefinition


T = TypeVar("T", bound="DefinitionsRPCCommand")


@_attrs_define
class DefinitionsRPCCommand:
    """
    Attributes:
        name (str):
        description (str):
        default_auth (DefinitionsRPCCommandDefaultAuth):
        request_params (list['DefinitionsFieldDefinition']):
        response_params (list['DefinitionsFieldDefinition']):
        depends_on (Union[Unset, str]):
        default (Union[Unset, str]):
        rpc_data (Union[Unset, bool]): Whether the command is an RPC data command
    """

    name: str
    description: str
    default_auth: DefinitionsRPCCommandDefaultAuth
    request_params: list["DefinitionsFieldDefinition"]
    response_params: list["DefinitionsFieldDefinition"]
    depends_on: Unset | str = UNSET
    default: Unset | str = UNSET
    rpc_data: Unset | bool = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description = self.description

        default_auth = self.default_auth.value

        request_params = []
        for request_params_item_data in self.request_params:
            request_params_item = request_params_item_data.to_dict()
            request_params.append(request_params_item)

        response_params = []
        for response_params_item_data in self.response_params:
            response_params_item = response_params_item_data.to_dict()
            response_params.append(response_params_item)

        depends_on = self.depends_on

        default = self.default

        rpc_data = self.rpc_data

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "description": description,
                "default_auth": default_auth,
                "request_params": request_params,
                "response_params": response_params,
            }
        )
        if depends_on is not UNSET:
            field_dict["depends_on"] = depends_on
        if default is not UNSET:
            field_dict["default"] = default
        if rpc_data is not UNSET:
            field_dict["rpc_data"] = rpc_data

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.definitions_field_definition import DefinitionsFieldDefinition

        d = dict(src_dict)
        name = d.pop("name")

        description = d.pop("description")

        default_auth = DefinitionsRPCCommandDefaultAuth(d.pop("default_auth"))

        request_params = []
        _request_params = d.pop("request_params")
        for request_params_item_data in _request_params:
            request_params_item = DefinitionsFieldDefinition.from_dict(request_params_item_data)

            request_params.append(request_params_item)

        response_params = []
        _response_params = d.pop("response_params")
        for response_params_item_data in _response_params:
            response_params_item = DefinitionsFieldDefinition.from_dict(response_params_item_data)

            response_params.append(response_params_item)

        depends_on = d.pop("depends_on", UNSET)

        default = d.pop("default", UNSET)

        rpc_data = d.pop("rpc_data", UNSET)

        definitions_rpc_command = cls(
            name=name,
            description=description,
            default_auth=default_auth,
            request_params=request_params,
            response_params=response_params,
            depends_on=depends_on,
            default=default,
            rpc_data=rpc_data,
        )

        definitions_rpc_command.additional_properties = d
        return definitions_rpc_command

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3

import enum


class WiFiState(enum.IntEnum):
    DISCONNECTED = 0
    INTERFACE_DISABLED = 1
    INACTIVE = 2
    SCANNING = 3
    AUTHENTICATING = 4
    ASSOCIATING = 5
    ASSOCIATED = 6
    HANDSHAKE_4WAY = 7
    GROUP_HANDSHAKE = 8
    COMPLETED = 9
    UNKNOWN = 10


class InterfaceMode(enum.IntEnum):
    INFRASTRUCTURE_STATION = 0
    IBSS_STATION = 1
    AP = 2
    P2P_GO = 3
    P2P_GROUP_FORMATION = 4
    MESH = 5


class FrequencyBand(enum.IntEnum):
    BAND_2_4_GHZ = 0
    BAND_5_GHZ = 1
    BAND_6_GHZ = 2

    def __str__(self):
        pretty_names = {
            self.BAND_2_4_GHZ: "2.4 GHz",
            self.BAND_5_GHZ: "5 GHz",
            self.BAND_6_GHZ: "6 GHz",
        }
        return pretty_names[self]


class FrequencyChannel(enum.IntEnum):
    CHANNEL_ANY = 255


class LinkMode(enum.IntEnum):
    WIFI_802_11 = 0
    WIFI_802_11b = 1
    WIFI_802_11a = 2
    WIFI_802_11g = 3
    WIFI_802_11n = 4
    WIFI_802_11ac = 5
    WIFI_802_11ax = 6
    WIFI_802_11ax_6Ghz = 7
    WIFI_802_11be = 8

    def __str__(self):
        pretty_names = {
            self.WIFI_802_11: "802.11 (Legacy)",
            self.WIFI_802_11b: "802.11.b",
            self.WIFI_802_11a: "802.11.a",
            self.WIFI_802_11g: "802.11.g",
            self.WIFI_802_11n: "802.11.n",
            self.WIFI_802_11ac: "802.11.ac",
            self.WIFI_802_11ax: "802.11.ax",
            self.WIFI_802_11ax_6Ghz: "802.11.ax (6 GHz)",
            self.WIFI_802_11be: "802.11.be",
        }
        return pretty_names[self]


class SecurityType(enum.IntEnum):
    NONE = 0
    WPA2_PSK = 1
    WPA2_PSK_SHA256 = 2
    WPA3_SAE = 3
    GB_WAPI = 4
    EAP = 5
    WEP = 6
    WPA_PSK = 7

    def __str__(self):
        pretty_names = {
            self.NONE: "None",
            self.WPA2_PSK: "WPA2-PSK",
            self.WPA2_PSK_SHA256: "WPA2-PSK-SHA256",
            self.WPA3_SAE: "WPA3-SAE",
            self.GB_WAPI: "GB 15629.11-2003 WAPI",
            self.EAP: "EAP",
            self.WEP: "WEP",
            self.WPA_PSK: "WPA-PSK",
        }
        return pretty_names[self]



#!/usr/bin/env python3

"""Run a local server for TDF viewing"""

__author__ = "Jordan Yates"
__copyright__ = "Copyright 2024, Embeint Holdings Pty Ltd"

import asyncio
import pathlib
import threading
import time
from typing import Any

from aiohttp import web
from aiohttp.web_request import BaseRequest
from aiohttp.web_runner import GracefulExit

import infuse_iot.epacket.interface as interface
from infuse_iot.commands import InfuseCommand
from infuse_iot.common import InfuseType
from infuse_iot.definitions.tdf import structs
from infuse_iot.generated.tdf_base import TdfStructBase
from infuse_iot.socket_comms import (
    ClientNotificationEpacketReceived,
    LocalClient,
    default_multicast_address,
)
from infuse_iot.tdf import TDF
from infuse_iot.time import InfuseTime
from infuse_iot.util.console import Console
from infuse_iot.util.threading import SignaledThread


class SubCommand(InfuseCommand):
    NAME = "localhost"
    HELP = "Run a local server for TDF viewing"
    DESCRIPTION = "Run a local server for TDF viewing"

    @classmethod
    def add_parser(cls, parser):
        parser.add_argument("--port", "-p", type=int, default=8080, help="Port number for localhost server")

    def __init__(self, args):
        self._data_lock = threading.Lock()
        self._columns: dict[str, dict] = {}
        self._data: dict[int, dict] = {}
        self._port: int = args.port

        self._client = LocalClient(default_multicast_address(), 1.0)
        self._decoder = TDF()

    # Serve the HTML file
    async def handle_index(self, _request):
        this_folder = pathlib.Path(__file__).parent

        return web.FileResponse(this_folder / "localhost" / "index.html")

    async def websocket_handler(self, request: BaseRequest):
        ws = web.WebSocketResponse()
        await ws.prepare(request)

        Console.log_info(f"Websocket client connected ({request.remote})")

        try:
            while True:
                # Example data sent to the client
                self._data_lock.acquire(blocking=True)
                columns = [
                    {
                        "title": "Metadata",
                        "headerHozAlign": "center",
                        "frozen": True,
                        "columns": [
                            {
                                "title": "Device",
                                "field": "infuse_id",
                                "headerHozAlign": "center",
                            },
                            {
                                "title": "Last Heard",
                                "field": "time",
                                "headerHozAlign": "center",
                            },
                            {
                                "title": "Bluetooth",
                                "headerHozAlign": "center",
                                "columns": [
                                    {
                                        "title": "Address",
                                        "field": "bt_addr",
                                        "headerHozAlign": "center",
                                    },
                                    {
                                        "title": "RSSI (dBm)",
                                        "field": "bt_rssi",
                                        "headerVertical": "flip",
                                        "hozAlign": "right",
                                    },
                                ],
                            },
                        ],
                    }
                ]
                # Put ANNOUNCE TDF first, if it exists
                if "ANNOUNCE" in self._columns:
                    sorted_tdfs = ["ANNOUNCE"] + [v for v in sorted(self._columns) if v != "ANNOUNCE"]
                else:
                    sorted_tdfs = sorted(self._columns)

                for tdf_name in sorted_tdfs:
                    columns.append(
                        {
                            "title": tdf_name,
                            "field": tdf_name,
                            "columns": self._columns[tdf_name],
                            "headerHozAlign": "center",
                        }
                    )
                devices = sorted(self._data.keys())
                message = {
                    "columns": columns,
                    "rows": [self._data[d] for d in devices],
                    "tdfs": list(self._columns.keys()),
                }
                self._data_lock.release()

                await ws.send_json(message)
                await asyncio.sleep(1)
        except (asyncio.CancelledError, ConnectionResetError):
            pass
        finally:
            await ws.close()

        Console.log_info(f"Websocket client disconnected ({request.remote})")
        return ws

    def tdf_columns(self, tdf):
        out = []

        def column_title(struct, name):
            postfix = struct._postfix_[name]
            if postfix != "":
                return f"{name} ({postfix})"
            return name

        for field in tdf.field_information():
            if field["type"] == structs.tdf_struct_mcuboot_img_sem_ver:
                # Special case version struct to make reading versions easier
                s = {
                    "title": column_title(tdf, field["name"]),
                    "field": f"{tdf.NAME}.{field['name']}",
                    "headerVertical": "flip",
                    "hozAlign": "right",
                }
            elif "subfields" in field:
                sub: list[dict[str, Any]] = []
                for subfield in field["subfields"]:
                    sub.append(
                        {
                            "title": column_title(field["type"], subfield["name"]),
                            "field": f"{tdf.NAME}.{field['name']}.{subfield['name']}",
                            "headerVertical": "flip",
                            "hozAlign": "right",
                        }
                    )
                s = {"title": field["name"], "headerHozAlign": "center", "columns": sub}
            else:
                s = {
                    "title": column_title(tdf, field["name"]),
                    "field": f"{tdf.NAME}.{field['name']}",
                    "headerVertical": "flip",
                    "hozAlign": "right",
                }
            out.append(s)
        return out

    def recv_thread(self) -> None:
        msg = self._client.receive()
        if msg is None:
            return
        if not isinstance(msg, ClientNotificationEpacketReceived):
            return
        if msg.epacket.ptype != InfuseType.TDF:
            return

        source = msg.epacket.route[0]

        self._data_lock.acquire(blocking=True)

        if source.infuse_id not in self._data:
            self._data[source.infuse_id] = {
                "infuse_id": f"0x{source.infuse_id:016x}",
            }
            # Set default application ID
            self._data[source.infuse_id]["ANNOUNCE"] = {"application": "Unknown"}

        self._data[source.infuse_id]["time"] = InfuseTime.utc_time_string(time.time())
        if source.interface == interface.ID.BT_ADV:
            addr_bytes = source.interface_address.val.addr_val.to_bytes(6, "big")
            addr_str = ":".join([f"{x:02x}" for x in addr_bytes])
            self._data[source.infuse_id]["bt_addr"] = addr_str
            self._data[source.infuse_id]["bt_rssi"] = source.rssi

        for tdf in self._decoder.decode(msg.epacket.payload):
            t = tdf.data[-1]
            if t.NAME not in self._columns:
                self._columns[t.NAME] = self.tdf_columns(t)
            if t.NAME not in self._data[source.infuse_id]:
                self._data[source.infuse_id][t.NAME] = {}

            for field in t.iter_fields(nested_iter=False):
                if isinstance(field.val, structs.tdf_struct_mcuboot_img_sem_ver):
                    # Special case version struct to make reading versions easier
                    val = f"{field.val.major}.{field.val.minor}.{field.val.revision}+{field.val.build_num:08x}"
                    self._data[source.infuse_id][t.NAME][field.field] = val
                elif isinstance(field.val, TdfStructBase):
                    for s in field.val.iter_fields(field.field):
                        if s.field not in self._data[source.infuse_id][t.NAME]:
                            self._data[source.infuse_id][t.NAME][s.field] = {}
                        self._data[source.infuse_id][t.NAME][s.field][s.subfield] = s.val_fmt()
                else:
                    self._data[source.infuse_id][t.NAME][field.field] = field.val_fmt()
        self._data_lock.release()

    def run(self):
        Console.init()
        app = web.Application()
        # Route for serving the HTML file
        app.router.add_get("/", self.handle_index)
        # Route for WebSocket
        app.router.add_get("/ws", self.websocket_handler)

        rx_thread = SignaledThread(self.recv_thread)
        rx_thread.start()

        # Run server
        try:
            web.run_app(app, host="localhost", port=self._port)
        except GracefulExit:
            pass
        finally:
            rx_thread.stop()
        rx_thread.join(1.0)



from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="GenerateMQTTTokenBody")


@_attrs_define
class GenerateMQTTTokenBody:
    """
    Attributes:
        organisation_id (UUID): ID of organisation to scope the token to
        ttl_seconds (int):  Default: 3600.
    """

    organisation_id: UUID
    ttl_seconds: int = 3600
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        organisation_id = str(self.organisation_id)

        ttl_seconds = self.ttl_seconds

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "organisationId": organisation_id,
                "ttlSeconds": ttl_seconds,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        organisation_id = UUID(d.pop("organisationId"))

        ttl_seconds = d.pop("ttlSeconds")

        generate_mqtt_token_body = cls(
            organisation_id=organisation_id,
            ttl_seconds=ttl_seconds,
        )

        generate_mqtt_token_body.additional_properties = d
        return generate_mqtt_token_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3

import tabulate

import infuse_iot.definitions.rpc as defs
from infuse_iot.commands import InfuseRpcCommand
from infuse_iot.zephyr.errno import errno


class heap_stats(InfuseRpcCommand, defs.heap_stats):
    @classmethod
    def add_parser(cls, parser):
        pass

    def __init__(self, args):
        self.args = args

    def request_struct(self):
        return self.request()

    def request_json(self):
        return {}

    def handle_response(self, return_code, response):
        if return_code != 0:
            print(f"Failed to query heap stats ({errno.strerror(-return_code)})")
            return

        stats = []
        for heap in response.stats:
            total = heap.free_bytes + heap.allocated_bytes
            current_percent = 100 * (heap.allocated_bytes / total)
            max_percent = 100 * (heap.max_allocated_bytes / total)
            stats.append(
                [
                    f"0x{heap.addr:08x}",
                    total,
                    heap.allocated_bytes,
                    f"{current_percent:4.1f}%",
                    heap.max_allocated_bytes,
                    f"{max_percent:4.1f}%",
                ]
            )

        print(tabulate.tabulate(stats, headers=["Address", "Total Size", "Current Usage", "", "Max Usage", ""]))



#!/usr/bin/env python3

import ctypes

import tabulate

from infuse_iot.commands import InfuseRpcCommand
from infuse_iot.definitions import rpc as rpc_defs
from infuse_iot.definitions import tdf as tdf_defs
from infuse_iot.util.ctypes import VLACompatLittleEndianStruct
from infuse_iot.zephyr.errno import errno


class zbus_channel_state(InfuseRpcCommand, rpc_defs.zbus_channel_state):
    class response(VLACompatLittleEndianStruct):
        _fields_ = [
            ("pub_timestamp", ctypes.c_uint64),
            ("pub_count", ctypes.c_uint32),
            ("pub_period_ms", ctypes.c_uint32),
        ]
        vla_field = ("data", 0 * ctypes.c_byte)

    class BatteryChannel:
        id = 0x43210000
        data = tdf_defs.readings.battery_state

    class AmbeintEnvChannel(ctypes.LittleEndianStructure):
        id = 0x43210001
        data = tdf_defs.readings.ambient_temp_pres_hum

    class ImuChannel(ctypes.LittleEndianStructure):
        id = 0x43210002
        data = None

    class AccMagChannel(ctypes.LittleEndianStructure):
        id = 0x43210003
        data = None

    class LocationChannel(ctypes.LittleEndianStructure):
        id = 0x43210004
        data = tdf_defs.readings.gcs_wgs84_llha

    class NavPvtUbxChannel(ctypes.LittleEndianStructure):
        id = 0x43210007
        data = tdf_defs.readings.ubx_nav_pvt

    class NavPvtNRFChannel(ctypes.LittleEndianStructure):
        id = 0x43210008
        data = tdf_defs.readings.nrf9x_gnss_pvt

    @classmethod
    def add_parser(cls, parser):
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            "--battery",
            dest="channel",
            action="store_const",
            const=cls.BatteryChannel,
            help="Battery channel",
        )
        group.add_argument(
            "--ambient-env",
            dest="channel",
            action="store_const",
            const=cls.AmbeintEnvChannel,
            help="Ambient environmental channel",
        )
        group.add_argument(
            "--imu",
            dest="channel",
            action="store_const",
            const=cls.ImuChannel,
            help="IMU channel",
        )
        group.add_argument(
            "--acc-mag",
            dest="channel",
            action="store_const",
            const=cls.AccMagChannel,
            help="Accelerometer magnitude channel",
        )
        group.add_argument(
            "--location",
            dest="channel",
            action="store_const",
            const=cls.LocationChannel,
            help="Location channel",
        )
        group.add_argument(
            "--nav-pvt-ubx",
            dest="channel",
            action="store_const",
            const=cls.NavPvtUbxChannel,
            help="ublox NAV-PVT channel",
        )
        group.add_argument(
            "--nav-pvt-nrf",
            dest="channel",
            action="store_const",
            const=cls.NavPvtNRFChannel,
            help="nRF9x NAV-PVT channel",
        )

    def __init__(self, args):
        self._channel = args.channel

    def request_struct(self):
        return self.request(self._channel.id)

    def request_json(self):
        return {"channel_id": str(self._channel.id)}

    def handle_response(self, return_code, response):
        if return_code != 0:
            print(f"Failed to query channel ({errno.strerror(-return_code)})")
            return

        from infuse_iot.time import InfuseTime

        pub_time = InfuseTime.unix_time_from_epoch(response.pub_timestamp)
        data_bytes = bytes(response.data)

        print(f"\t  Publish time: {InfuseTime.utc_time_string(pub_time)}")
        print(f"\t Publish count: {response.pub_count}")
        print(f"\tPublish period: {response.pub_period_ms} ms")
        try:
            if self._channel.data is None:
                print(f"\t          Data: {data_bytes.hex()}")
            else:
                data = self._channel.data.from_buffer_copy(data_bytes)
                table = []
                for field in data.iter_fields():
                    table.append([field.name, field.val_fmt(), field.postfix])
                print(tabulate.tabulate(table, tablefmt="simple"))
        except Exception as _:
            print(f"\t          Data: {data_bytes.hex()}")
